<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Components\BusinessLogicServices\Helpers\RecipientHandler;

use CleverReach\WooCommerce\Components\BusinessLogicServices\Order_Items_Service;
use CleverReach\WooCommerce\Components\BusinessLogicServices\Recipient_Service;
use CleverReach\WooCommerce\Components\InfrastructureServices\Config_Service;
use CleverReach\WooCommerce\Components\Repositories\Roles_Repository;
use CleverReach\WooCommerce\Components\Utility\Database;
use CleverReach\WooCommerce\Components\Utility\Shop_Helper;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\Recipient;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\SpecialTag;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\SpecialTagCollection;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\Tag;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\TagCollection;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Interfaces\OrderItems;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;

/**
 * Class Customer_Handler
 *
 * @package CleverReach\WooCommerce\Components\BusinessLogicServices\Helpers\RecipientHandler
 */
class Customer_Handler extends Recipient_Handler {
	/**
	 * Role repository.
	 *
	 * @var Roles_Repository
	 */
	private $role_repository;

	/**
	 * Generates properly formatted CleverReach recipient.
	 *
	 * @param string $id Customer ID.
	 * @param bool   $include_orders Should response include orders with customer.
	 *
	 * @return Recipient | null
	 *
	 * @throws \Exception Exception.
	 */
	public function handle( $id, $include_orders ) {
		/**
		 * WordPress user.
		 *
		 * @var \WP_User
		 */
		$customer = get_user_by( 'ID', $id );

		if ( empty( $customer ) || empty( $customer->user_email ) ) {
			return null;
		}

		$customer_meta = get_userdata( $id );

		$address1 = get_user_meta( $customer->ID, 'billing_address_1', true );
		$address2 = get_user_meta( $customer->ID, 'billing_address_2', true );

		$street = ( ! empty( $address1 ) ? $address1 : '' ) . ( ! empty( $address2 ) ? ' ' . $address2 : '' );
		$street = trim( $street );

		$language = $this->get_user_language( $id );

		// Country code (ex. DE, FR, IT...).
		$country = get_user_meta( $customer->ID, 'billing_country', true );

		$recipient = new Recipient( $customer->user_email );
		$recipient->setRegistered( new \DateTime( $customer_meta->user_registered ) );
		$recipient->setFirstName( $this->get_first_name( $customer ) );
		$recipient->setLastName( $customer->last_name );
		$recipient->setSource( Shop_Helper::get_shop_url() );
		$recipient->setShop( Shop_Helper::get_shop_name() );
		$recipient->setCustomerNumber( $id );
		$recipient->setLanguage( $language );
		$recipient->setStreet( $street );
		$recipient->setZip( get_user_meta( $customer->ID, 'billing_postcode', true ) );
		$recipient->setCity( get_user_meta( $customer->ID, 'billing_city', true ) );
		$recipient->setState( get_user_meta( $customer->ID, 'billing_state', true ) );
		$recipient->setCountry( ! empty( $country ) ? wc()->countries->countries[ $country ] : '' );
		$recipient->setCompany( get_user_meta( $customer->ID, 'billing_company', true ) );
		$recipient->setPhone( get_user_meta( $customer->ID, 'billing_phone', true ) );
		$recipient->setInternalId( Recipient_Service::CUSTOMER_ID_PREFIX . $id );

		$is_subscriber = $this->get_recipient_newsletter_status( $id );

		$recipient->setNewsletterSubscription( $is_subscriber );
		$recipient->setActive( $is_subscriber );
		$recipient->setTags( $this->format_tags( $customer->roles ) );
		$recipient->setSpecialTags( $this->format_special_tags( $customer, $is_subscriber ) );
		/**
		 * Order items service.
		 *
		 * @var Order_Items_Service $order_items_service
		 */
		$order_items_service = ServiceRegister::getService( OrderItems::CLASS_NAME );
		$order_items         = $order_items_service->get_customer_order_items( $customer->ID );

		if ( ! empty( $order_items ) ) {
			$recipient->setLastOrderDate( $order_items[0]->getStamp() );
			if ( $include_orders ) {
				$recipient->setOrders( $order_items );
			}
		}

		return $recipient;
	}

	/**
	 * Returns newsletter subscription status for a recipient with provided ID.
	 *
	 * @param int $user_id User ID.
	 *
	 * @return bool
	 */
	private function get_recipient_newsletter_status( $user_id ) {
		global $wpdb;

		$database_handler       = new Database( $wpdb );
		$newsletter_column      = $database_handler->get_newsletter_column();
		$user_newsletter_status = get_user_meta( $user_id, $newsletter_column, true );

		if ( null !== $user_newsletter_status && '' !== $user_newsletter_status ) {
			return '1' === $user_newsletter_status;
		}

		$default_newsletter_status_handling = $this->get_config_service()->get_default_newsletter_status();

		return '1' === $default_newsletter_status_handling;
	}

	/**
	 * Formats customer tags.
	 *
	 * @param array $customer_roles Role of the customer.
	 *
	 * @return TagCollection
	 */
	private function format_tags( $customer_roles ) {
		$tags = new TagCollection();

		$all_roles  = $this->get_role_repository()->get_all_roles();
		$user_roles = array_intersect_key( $all_roles, array_flip( $customer_roles ) );

		foreach ( $user_roles as $role ) {
			$tags->addTag( new Tag( $role, Recipient_Service::TAG_TYPE_ROLE ) );
		}

		$tags->addTag( new Tag( Shop_Helper::get_shop_name() ?: Shop_Helper::get_shop_url(), Recipient_Service::TAG_TYPE_SITE ) );

		return $tags;
	}

	/**
	 * Formats customer special tags.
	 *
	 * @param \WP_User $customer WooCommerce customer.
	 *
	 * @param bool     $is_subscriber Flag that specifies whether user is subscriber.
	 *
	 * @return SpecialTagCollection
	 */
	private function format_special_tags( $customer, $is_subscriber ) {
		$special_tags = new SpecialTagCollection();

		$special_tags->addTag( SpecialTag::customer() );

		if ( wc_get_customer_order_count( $customer->ID ) > 0 ) {
			$special_tags->addTag( SpecialTag::buyer() );
		}

		if ( $is_subscriber ) {
			$special_tags->addTag( SpecialTag::subscriber() );
		}

		return $special_tags;
	}

	/**
	 * Returns config service.
	 *
	 * @return Config_Service
	 */
	private function get_config_service() {
		return ServiceRegister::getService( Configuration::CLASS_NAME );
	}

	/**
	 * Retrieves customer first name.
	 *
	 * @param \WP_User $user WordPress user.
	 *
	 * @return string
	 */
	private function get_first_name( $user ) {
		if ( ! empty( $user->first_name ) ) {
			return $user->first_name;
		}

		if ( ! empty( $user->nickname ) ) {
			return $user->nickname;
		}

		if ( ! empty( $user->user_login ) ) {
			return $user->user_login;
		}

		return '';
	}

	/**
	 * User language in format en-GB, de-DE etc.
	 *
	 * @param int $user_id User id.
	 *
	 * @return string
	 */
	private function get_user_language( $user_id ) {
		$locale = get_user_locale( $user_id );

		if ( empty( $locale ) ) {
			$locale = get_option( 'WPLANG' );
		}

		require_once( ABSPATH . 'wp-admin/includes/translation-install.php' );
		$translations = wp_get_available_translations();

		return isset( $translations[ $locale ] ) ?
			$translations[ $locale ]['native_name'] :
			( function_exists('locale_get_display_language') ? locale_get_display_language( $locale ) : '');
	}

	/**
	 * Retrieves recipient repository.
	 *
	 * @return Roles_Repository
	 */
	private function get_role_repository() {
		if ( null === $this->role_repository ) {
			$this->role_repository = new Roles_Repository();
		}

		return $this->role_repository;
	}
}
