<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Components\BusinessLogicServices\Helpers\RecipientHandler;

use CleverReach\WooCommerce\Components\BusinessLogicServices\Order_Items_Service;
use CleverReach\WooCommerce\Components\BusinessLogicServices\Recipient_Service;
use CleverReach\WooCommerce\Components\Repositories\Order_Repository;
use CleverReach\WooCommerce\Components\Utility\Database;
use CleverReach\WooCommerce\Components\Utility\Shop_Helper;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\Recipient;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\SpecialTag;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\SpecialTagCollection;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\Tag;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\TagCollection;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Interfaces\OrderItems;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;

/**
 * Class Guest_Handler
 *
 * @package CleverReach\WooCommerce\Components\BusinessLogicServices\Helpers\RecipientHandler
 */
class Guest_Handler extends Recipient_Handler {

	/**
	 * Generates properly formatted CleverReach recipient.
	 *
	 * @param string $id Guest order ID.
	 * @param bool   $include_orders Should response include orders with recipient.
	 *
	 * @return Recipient
	 *
	 * @throws \Exception Exception.
	 */
	public function handle( $id, $include_orders ) {
		global $wpdb;
		$database         = new Database( $wpdb );
		$order_repository = new Order_Repository();
		$guest            = $order_repository->get_guest_by_id( $id );
		if ( empty( $guest['email'] ) || 0 === $order_repository->get_guest_order_count( $guest['email'] ) ) {
			return null;
		}

		$street = ( ! empty( $guest['address_1'] ) ? $guest['address_1'] : '' )
		. ( ! empty( $guest['address_2'] ) ? ' ' . $guest['address_2'] : '' );

		$newsletter_subscription = (int) get_post_meta( $id, $database->get_newsletter_column(), true ) === 1;

		$recipient = new Recipient( $guest['email'] );
		$recipient->setRegistered(
			! empty( $guest['registered'] )
			? new \DateTime( $guest['registered'] )
			: null
		);
		$recipient->setFirstName( ! empty( $guest['first_name'] ) ? $guest['first_name'] : '' );
		$recipient->setLastName( ! empty( $guest['last_name'] ) ? $guest['last_name'] : '' );
		$recipient->setSource( Shop_Helper::get_shop_url() );
		$recipient->setShop( Shop_Helper::get_shop_name() );
		$recipient->setStreet( $street );
		$recipient->setZip( ! empty( $guest['postcode'] ) ? $guest['postcode'] : '' );
		$recipient->setCity( ! empty( $guest['city'] ) ? $guest['city'] : '' );
		$recipient->setState( ! empty( $guest['state'] ) ? $guest['state'] : '' );
		$recipient->setCountry( ! empty( $guest['country'] ) ? wc()->countries->countries[ $guest['country'] ] : '' );
		$recipient->setPhone( ! empty( $guest['phone'] ) ? $guest['phone'] : '' );
		$recipient->setNewsletterSubscription( $newsletter_subscription );
		$recipient->setActive( $newsletter_subscription );
		$recipient->setTags( $this->format_tags() );
		$recipient->setSpecialTags( $this->format_special_tags( $newsletter_subscription ) );
		$recipient->setInternalId( Recipient_Service::GUEST_ID_PREFIX . $id );

		$order_ids = $order_repository->get_guest_orders( $guest['email'] );
		if ( ! empty( $order_ids ) ) {
			/**
			 * Order item service.
			 *
			 * @var Order_Items_Service $order_item_service
			 */
			$order_item_service = ServiceRegister::getService( OrderItems::CLASS_NAME );
			$order_items        = $order_item_service->getOrderItems( $order_ids );

			if ( ! empty( $order_items ) ) {
				$recipient->setLastOrderDate( $order_items[0]->getStamp() );

				if ( $include_orders ) {
					$recipient->setOrders( $order_items );
				}
			}
		}

		return $recipient;
	}

	/**
	 * Formats guest tags.
	 *
	 * @return TagCollection
	 */
	private function format_tags() {
		$tags = new TagCollection();

		$tags->addTag( new Tag( Recipient_Service::GUEST_CUSTOMER_TAG, Recipient_Service::TAG_TYPE_ROLE ) );
		$tags->addTag( new Tag( Shop_Helper::get_shop_name() ?: Shop_Helper::get_shop_url(), Recipient_Service::TAG_TYPE_SITE ) );

		return $tags;
	}

	/**
	 * Formats guest special tags.
	 *
	 * @param bool $is_subscriber Whether this guest has subscribed to newsletter.
	 *
	 * @return SpecialTagCollection
	 */
	private function format_special_tags( $is_subscriber ) {
		$special_tags = new SpecialTagCollection();

		$special_tags->addTag( SpecialTag::customer() );
		$special_tags->addTag( SpecialTag::buyer() );

		if ( $is_subscriber ) {
			$special_tags->addTag( SpecialTag::subscriber() );
		}

		return $special_tags;
	}
}
