<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Components\BusinessLogicServices;

use CleverReach\WooCommerce\Components\BusinessLogicServices\Helpers\RecipientHandler\Customer_Handler;
use CleverReach\WooCommerce\Components\BusinessLogicServices\Helpers\RecipientHandler\Guest_Handler;
use CleverReach\WooCommerce\Components\BusinessLogicServices\Helpers\RecipientHandler\Recipient_Handler;
use CleverReach\WooCommerce\Components\Repositories\Recipient_Repository;
use CleverReach\WooCommerce\Components\Repositories\Roles_Repository;
use CleverReach\WooCommerce\Components\Utility\Shop_Helper;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\SpecialTag;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\SpecialTagCollection;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\Tag;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\TagCollection;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Interfaces\Recipients;

/**
 * Class Recipient_Service
 *
 * @package CleverReach\WooCommerce\Components\BusinessLogicServices
 */
class Recipient_Service implements Recipients {
	const TAG_TYPE_ROLE      = 'Role';
	const TAG_TYPE_SITE      = 'Site';
	const GUEST_CUSTOMER_TAG = 'GuestCustomer';

	const CUSTOMER_ID_PREFIX = 'C-';
	const GUEST_ID_PREFIX    = 'G-';

	/**
	 * Roles repository.
	 *
	 * @var Roles_Repository
	 */
	protected $roles_repository;
	/**
	 * Recipients repository.
	 *
	 * @var Recipient_Repository
	 */
	protected $recipients_repository;
	/**
	 * Array of recipient handlers.
	 *
	 * @var array
	 */
	protected $recipient_handlers;

	/**
	 * Recipient_Service constructor.
	 */
	public function __construct() {
		$this->recipient_handlers = array(
			'C' => new Customer_Handler(),
			'G' => new Guest_Handler(),
		);
	}

	/**
	 * Gets all tags as a collection.
	 *
	 * @return \CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\TagCollection
	 */
	public function getAllTags() {
		$tag_collection = new TagCollection();
		$tag_collection->add( $this->get_formatted_tags( $this->get_roles_repository()->get_all_roles(), self::TAG_TYPE_ROLE ) );
		$tag_collection->addTag( new Tag( Shop_Helper::get_shop_name() ?: Shop_Helper::get_shop_url(), self::TAG_TYPE_SITE ) );
		$tag_collection->addTag( new Tag( self::GUEST_CUSTOMER_TAG, self::TAG_TYPE_ROLE ) );

		return $tag_collection;
	}

	/**
	 * Gets all special tags as a collection.
	 *
	 * @return \CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\SpecialTagCollection
	 */
	public function getAllSpecialTags() {
		return new SpecialTagCollection(
			array(
				SpecialTag::subscriber(),
				SpecialTag::customer(),
				SpecialTag::buyer(),
			)
		);
	}

	/**
	 * Gets CleverReach recipients with tags, properly formatted.
	 *
	 * @param array $batch_recipient_ids Batch of WP_User ids prefixed with C- (regular registered customers) and
	 * WC_Order ids prefixed with G- (this represents guest customers).
	 *
	 * @param bool  $include_orders Specifies whether to include orders in result or not.
	 *
	 * @return array
	 */
	public function getRecipientsWithTags( array $batch_recipient_ids, $include_orders ) {
		$result = array();

		foreach ( $batch_recipient_ids as $complex_id ) {
			list( $prefix, $id ) = explode( '-', $complex_id );

			$recipient = $this->get_recipient_handler( $prefix )->handle( $id, $include_orders );
			if ( null !== $recipient ) {
				$result[] = $recipient;
			}
		}

		return $result;
	}

	/**
	 * Gets all recipients IDs from source system.
	 *
	 * @return array of strings
	 */
	public function getAllRecipientsIds() {
		$repository = $this->get_recipients_repository();
		$customers  = $repository->get_all_customer_ids();
		$guests     = $repository->get_all_guest_ids();

		$formatted_customers = $this->format_entity_ids( $customers, self::CUSTOMER_ID_PREFIX );
		$formatted_guests    = $this->format_entity_ids( $guests, self::GUEST_ID_PREFIX );

		return array_merge( $formatted_customers, $formatted_guests );
	}

	/**
	 * Informs service about completed synchronization of provided recipients (IDs).
	 *
	 * @param array $recipient_ids Array of recipient IDs.
	 */
	public function recipientSyncCompleted( array $recipient_ids ) {
		// Nothing to do.
	}

	/**
	 * Generates properly formatted tag collection.
	 *
	 * @param array  $tags Array of tags to be formatted.
	 * @param string $tag_type Tag type.
	 *
	 * @return TagCollection
	 */
	protected function get_formatted_tags( array $tags, $tag_type ) {
		$tag_collection = new TagCollection();

		foreach ( $tags as $tag_name ) {
			$tag = new Tag( $tag_name, $tag_type );
			$tag_collection->addTag( $tag );
		}

		return $tag_collection;
	}

	/**
	 * Returns roles repository.
	 *
	 * @return Roles_Repository
	 */
	protected function get_roles_repository() {
		if ( null === $this->roles_repository ) {
			$this->roles_repository = new Roles_Repository();
		}

		return $this->roles_repository;
	}

	/**
	 * Returns recipient repository.
	 *
	 * @return Recipient_Repository
	 */
	protected function get_recipients_repository() {
		if ( null === $this->recipients_repository ) {
			$this->recipients_repository = new Recipient_Repository();
		}

		return $this->recipients_repository;
	}

	/**
	 * Formats entity ids for internal processing.
	 *
	 * @param array  $ids Entity IDs.
	 * @param string $prefix Entity prefix.
	 *
	 * @return array
	 */
	protected function format_entity_ids( array $ids, $prefix ) {
		$result = array();

		if ( empty( $ids ) ) {
			return $result;
		}

		foreach ( $ids as $id ) {
			$result[] = $prefix . $id;
		}

		return $result;
	}

	/**
	 * Returns appropriate recipient handler for the provided prefix.
	 *
	 * @param string $prefix Entity prefix.
	 *
	 * @return Recipient_Handler
	 */
	protected function get_recipient_handler( $prefix ) {
		return $this->recipient_handlers[ $prefix ];
	}
}
