<?php

namespace CleverReach\WooCommerce\Components\HookHandlers;

use CleverReach\WooCommerce\Components\Utility\Database;
use CleverReach\WooCommerce\Components\Utility\Shop_Helper;
use CleverReach\WooCommerce\Components\Utility\Task_Queue;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Sync\RecipientDeactivateSyncTask;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Sync\RecipientSyncTask;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Logger\Logger;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Exceptions\QueueStorageUnavailableException;

/**
 * Class CustomerUpdatedHandler
 *
 * @package CleverReach\WooCommerce\Components\HookHandlers
 */
class Customer_Updated_Handler extends Hook_Handler {
	/**
	 * Handles user updated event.
	 *
	 * @param int      $user_id id of updated user.
	 *
	 * @param \WP_User $old_user_data old data.
	 */
	public function handle( $user_id, $old_user_data ) {
		if ( ! $this->should_handle_event() ) {
			return;
		}

		Logger::logInfo( 'Customer updated event detected. User id: ' . $user_id, 'Integration' );

		$newsletter_status    = ! empty( $_POST[ Database::NEWSLETTER_STATUS_FIELD ] ) ? 1 : 0;
		$recipient_repository = $this->get_recipient_repository();
		$recipient_repository->update_user_newsletter_field( $user_id, $newsletter_status );

		if ( ! $this->is_initial_sync_queued() ) {
			return;
		}

		$new_data = $recipient_repository->get_user_data_by_id( $user_id );
		if ( ! empty( $old_user_data->user_email ) && $old_user_data->user_email !== $new_data->user_email ) {
			try {
				Task_Queue::enqueue( new RecipientDeactivateSyncTask( array( $old_user_data->user_email ) ) );
			} catch ( QueueStorageUnavailableException $e ) {
				Logger::logError(
					'Cannot enqueue RecipientDeactivateSyncTask because: ' . $e->getMessage(),
					'Integration'
				);
			}
		}

		$customer_id = $this->get_customer_id_with_prefix( $user_id );
		try {
			Task_Queue::enqueue( new RecipientSyncTask( array( $customer_id ) ) );
		} catch ( QueueStorageUnavailableException $e ) {
			Logger::logError(
				'Cannot enqueue RecipientSyncTask because: ' . $e->getMessage(),
				'Integration'
			);
		}
	}

	/**
	 * Check whether event should be handled or not.
	 *
	 * @return bool
	 */
	protected function should_handle_event() {
		return Shop_Helper::is_plugin_enabled() && Shop_Helper::is_woocommerce_active();
	}
}
