<?php

namespace CleverReach\WooCommerce\Components\HookHandlers;

use CleverReach\WooCommerce\Components\BusinessLogicServices\Recipient_Service;
use CleverReach\WooCommerce\Components\Repositories\Base_Repository;
use CleverReach\WooCommerce\Components\Utility\Database;
use CleverReach\WooCommerce\Components\Utility\Task_Queue;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\Tag;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\TagCollection;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Sync\FilterSyncTask;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Sync\RecipientSyncTask;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Logger\Logger;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Exceptions\QueueStorageUnavailableException;

/**
 * Class OptionChangedHandler
 *
 * @package CleverReach\WooCommerce\Components\HookHandlers
 */
class Option_Changed_Handler extends Hook_Handler {
	/**
	 * Handles option changed event.
	 *
	 * @param string $option changed option.
	 *
	 * @param mixed  $old_value old value.
	 *
	 * @param mixed  $new_value new value.
	 */
	public function handle( $option, $old_value, $new_value ) {
		if ( ! is_string( $option ) || ! $this->should_handle_event() ) {
			return;
		}

		$site_option_name = 'blogname';
		$role_option_name = Base_Repository::get_table_name( Database::ROLES_TABLE );

		if ( $site_option_name === $option ) {
			$this->handle_store_name_change( $old_value, $new_value );
		} elseif ( $role_option_name === $option ) {
			$this->handle_roles_change( $old_value, $new_value );
		}
	}

	/**
	 * Handles store name changed event.
	 *
	 * @param string $old_name old site name.
	 *
	 * @param string $new_name new site name.
	 */
	private function handle_store_name_change( $old_name, $new_name ) {
		if ( $old_name !== $new_name ) {
			Logger::logInfo( "Site name change event detected. Name changed from $old_name to $new_name." );

			$tags_to_delete = new TagCollection();
			$tags_to_delete->addTag( new Tag( $old_name, Recipient_Service::TAG_TYPE_SITE ) );

			try {
				$recipient_ids = $this->get_recipients_service()->getAllRecipientsIds();
				Task_Queue::enqueue( new FilterSyncTask() );
				Task_Queue::enqueue( new RecipientSyncTask( $recipient_ids, $tags_to_delete ) );
			} catch ( \Exception $e ) {
				Logger::logError(
					'Could not handle site name change event becaue: ' . $e->getMessage(),
					'Integration'
				);
			}
		}
	}

	/**
	 * Handles role changed event.
	 *
	 * @param array $old_roles roles before change.
	 *
	 * @param array $new_roles roles after change.
	 */
	private function handle_roles_change( $old_roles, $new_roles ) {
		Logger::logInfo( 'Role changed event detected.' );

		$changed_role_keys = array();
		$tags_to_delete    = new TagCollection();

		$old_role_keys = array_keys( $old_roles );
		$new_role_keys = array_keys( $new_roles );

		$deleted_roles = array_diff( $old_role_keys, $new_role_keys );
		$added_roles   = array_diff_assoc( $new_role_keys, $old_role_keys );

		if ( ! empty( $deleted_roles ) ) {
			$changed_role_keys = array_merge( $deleted_roles );

			foreach ( $deleted_roles as $deleted_role ) {
				$tags_to_delete->addTag(
					new Tag( $old_roles[ $deleted_role ]['name'], Recipient_Service::TAG_TYPE_ROLE )
				);
			}
		}

		foreach ( $old_roles as $old_role_key => $old_role ) {
			if ( ! empty( $new_roles[ $old_role_key ] ) && $old_role['name'] !== $new_roles[ $old_role_key ]['name'] ) {
				$changed_role_keys[] = $old_role_key;
				$tags_to_delete->addTag( new Tag( $old_role['name'], Recipient_Service::TAG_TYPE_ROLE ) );
			}
		}

		try {
			if ( ! empty( $added_roles ) || ! empty( $changed_role_keys ) ) {
				Task_Queue::enqueue( new FilterSyncTask() );
			}

			if ( ! empty( $changed_role_keys ) ) {
				$user_ids     = $this->get_recipient_repository()->get_user_ids_for_roles( $changed_role_keys );

				if ( ! empty( $user_ids ) ) {
					$customer_ids = $this->get_ids_with_prefix( $user_ids, Recipient_Service::CUSTOMER_ID_PREFIX );

					Task_Queue::enqueue( new RecipientSyncTask( $customer_ids, $tags_to_delete ) );
				}
			}
		} catch ( QueueStorageUnavailableException $e ) {
			Logger::logError( 'Could not handle role changed event because: ' . $e->getMessage(), 'Integration' );
		}
	}

	/**
	 * Prefixes array of ids.
	 *
	 * @param array  $ids ids to be prefixed.
	 *
	 * @param string $prefix id prefix.
	 *
	 * @return array
	 */
	private function get_ids_with_prefix( $ids, $prefix ) {
		$result = array();

		foreach ( $ids as $id ) {
			$result[] = $prefix . $id;
		}

		return $result;
	}
}
