<?php

namespace CleverReach\WooCommerce\Components\HookHandlers;

use CleverReach\WooCommerce\Components\BusinessLogicServices\Recipient_Service;
use CleverReach\WooCommerce\Components\Repositories\Order_Repository;
use CleverReach\WooCommerce\Components\Utility\Task_Queue;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Sync\RecipientSyncTask;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Logger\Logger;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Exceptions\QueueStorageUnavailableException;
use CleverReach\WooCommerce\Components\Utility\Database;

/**
 * Class OrderCreatedHandler
 *
 * @package CleverReach\WooCommerce\Components\HookHandlers
 */
class Order_Created_Handler extends Hook_Handler {
	/**
	 * Handles order created event.
	 *
	 * @param int $order_id id of created order.
	 */
	public function handle( $order_id ) {
		$order = wc_get_order( $order_id );
		if ( ! $this->should_handle_event() || ! $this->should_handle_order_event( $order ) ) {
			return;
		}

		Logger::logInfo( 'Order created event detected. Order id: ' . $order_id );

		$this->check_and_set_mailing_id( $order );

		$user = $order->get_user();
		// Checks if user is registered customer or guest customer.
		if ( false === $user ) {
			global $wpdb;
			$database = new Database( $wpdb );
			$id       = Recipient_Service::GUEST_ID_PREFIX . $order->get_id();
			if ( 'checkout' !== get_post_meta( $order_id, '_created_via', true ) ) {
				update_post_meta(
					$order_id,
					$database->get_newsletter_column(),
					$this->get_guest_newsletter_status( $order->get_billing_email(), $database )
				);
			}
		} else {
			$id = Recipient_Service::CUSTOMER_ID_PREFIX . $user->ID;
		}

		try {
			Task_Queue::enqueue( new RecipientSyncTask( array( $id ), null, true ) );
			Task_Queue::wakeup();
			update_post_meta( $order_id, 'cleverreach_order_handled', true );
		} catch ( QueueStorageUnavailableException $e ) {
			Logger::logError( 'Cannot enqueue RecipientSyncTask because' . $e->getMessage(), 'Integration' );
		}
	}

	/**
	 * Returns newsletter status for guest customer that the created order belongs to.
	 *
	 * @param string   $email Billing email of guest customer.
	 * @param Database $database Database object.
	 *
	 * @return bool Guest customer newsletter status.
	 */
	private function get_guest_newsletter_status( $email, $database ) {
		$order_repository = new Order_Repository();
		$order_id         = $order_repository->get_last_guest_checkout_order( $email );
		if ( 0 === $order_id ) {
			return false;
		}

		return (bool) get_post_meta( $order_id, $database->get_newsletter_column(), true );
	}

	/**
	 * Checks if order event should be handled.
	 *
	 * @param \WC_Order $order WooCommerce order object.
	 *
	 * @return bool Returns true if event should be handled, otherwise returns false.
	 */
	private function should_handle_order_event( $order ) {
		if ( null === $order ) {
			return false;
		}

		$handled = (bool) $order->get_meta( 'cleverreach_order_handled' );

		return ! (
			in_array( $order->get_status(), array( 'auto-draft', 'draft' ), true )
			|| ( 0 === $order->get_customer_id() && '' === $order->get_billing_email() )
			|| $handled
		);
	}

	/**
	 * Checks whether mailing ID has been set in session and if so, sets that mailing ID on provided order.
	 *
	 * @param \WC_Order $order WooCommerce order object.
	 */
	private function check_and_set_mailing_id( $order ) {
		if ( null === wc()->session ) {
			return;
		}

		$mailing_id = wc()->session->get( 'cr_mailing_id' );

		if ( ! empty( $mailing_id ) ) {
			$order->update_meta_data( '_cr_mailing_id', $mailing_id );
			$order->save();
		}
	}
}
