<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Components\InfrastructureServices;

use CleverReach\WooCommerce\Components\Repositories\Process_Repository;
use CleverReach\WooCommerce\Components\Utility\Shop_Helper;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Interfaces\Exposed\Runnable;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Interfaces\Required\AsyncProcessStarter;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Interfaces\Required\HttpClient;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Logger\Logger;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Utility\GuidProvider;

/**
 * Class Async_Process_Starter_Service
 *
 * @package CleverReach\WooCommerce\Components\InfrastructureServices
 */
class Async_Process_Starter_Service implements AsyncProcessStarter {
	/**
	 * HTTP client service.
	 *
	 * @var HttpClient
	 */
	private $http_client_service;

	/**
	 * Starts async process.
	 *
	 * @param Runnable $runner Task runner.
	 *
	 * @throws \CleverReach\WooCommerce\IntegrationCore\Infrastructure\Utility\Exceptions\HttpRequestException HTTP request exception.
	 * @throws \CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Exceptions\ProcessStarterSaveException Process starter exception.
	 */
	public function start( Runnable $runner ) {
		$guid_provider = new GuidProvider();
		$guid          = trim( $guid_provider->generateGuid() );

		$this->save_guid_and_runner( $guid, $runner );
		$this->start_runner_asynchronously( $guid );
	}

	/**
	 * Gets URL for async process controller.
	 *
	 * @return string
	 * @throws \Exception Exception.
	 */
	public function get_async_process_started_base_url() {
		return $this->format_url( 0 );
	}

	/**
	 * Saves runner and guid to storage
	 *
	 * @param string   $guid Globally Unique Identifier.
	 * @param Runnable $runner Task runner.
	 *
	 * @throws \CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Exceptions\ProcessStarterSaveException Process starter exception.
	 */
	private function save_guid_and_runner( $guid, Runnable $runner ) {
		try {
			$process_repository = new Process_Repository();
			$process_repository->save( $guid, $runner );
		} catch ( \Exception $e ) {
			Logger::logError( $e->getMessage(), 'Integration' );
			throw new \CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Exceptions\ProcessStarterSaveException(
				$e->getMessage(),
				0,
				$e
			);
		}
	}

	/**
	 * Starts runnable asynchronously
	 *
	 * @param string $guid Globally Unique Identifier.
	 *
	 * @throws \CleverReach\WooCommerce\IntegrationCore\Infrastructure\Utility\Exceptions\HttpRequestException HTTP request exception.
	 */
	private function start_runner_asynchronously( $guid ) {
		try {
			$this->get_http_client()->requestAsync( 'POST', $this->format_url( $guid ) );
		} catch ( \Exception $e ) {
			Logger::logError( $e->getMessage(), 'Integration' );
			throw new \CleverReach\WooCommerce\IntegrationCore\Infrastructure\Utility\Exceptions\HttpRequestException(
				$e->getMessage(),
				0,
				$e
			);
		}
	}

	/**
	 * Returns async process starter URL.
	 *
	 * @param string $guid Globally Unique Identifier.
	 *
	 * @return string
	 */
	private function format_url( $guid ) {
		$url = Shop_Helper::get_controller_url( 'Async_Process', 'run', array( 'guid' => $guid ) );

		return $url;
	}

	/**
	 * Returns HTTP Client service.
	 *
	 * @return HttpClient
	 */
	private function get_http_client() {
		if ( null === $this->http_client_service ) {
			$this->http_client_service = ServiceRegister::getService( HttpClient::CLASS_NAME );
		}

		return $this->http_client_service;
	}
}
