<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Components\InfrastructureServices;

use CleverReach\WooCommerce\Components\Utility\Shop_Helper;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Interfaces\Required\Configuration as ConfigInterface;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Interfaces\Required\ConfigRepositoryInterface;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\Notification;

/**
 * Class Config_Service
 *
 * @package CleverReach\WooCommerce\Components\InfrastructureServices
 */
class Config_Service extends ConfigInterface {
	const INTEGRATION_NAME = 'WooCommerce';
	const CLIENT_ID        = 'Hk9nekrQC5';
	const CLIENT_SECRET    = '2Xb5EabEugq1qmFukkG4Rop0MDlduXjx';

	/**
	 * Retrieves integration name
	 *
	 * @return string
	 */
	public function getIntegrationName() {
		return self::INTEGRATION_NAME;
	}

	/**
	 * Gets the name of CleverReach group/list that will be created.
	 *
	 * @return string
	 *   List name.
	 */
	public function getIntegrationListName() {
		return $this->getIntegrationName() . ' - ' . Shop_Helper::get_shop_name();
	}

	/**
	 * Returns queue name
	 *
	 * @return string
	 */
	public function getQueueName() {
		return 'WooCommerceDefault';
	}

	/**
	 * Returns client ID.
	 *
	 * @return string
	 */
	public function getClientId() {
		return self::CLIENT_ID;
	}

	/**
	 * Returns client secret.
	 *
	 * @return string
	 */
	public function getClientSecret() {
		return self::CLIENT_SECRET;
	}

	/**
	 * Return whether product search is enabled or not
	 *
	 * @return bool
	 */
	public function isProductSearchEnabled() {
		return true;
	}

	/**
	 * Retrieves parameters needed for product search registrations
	 */
	public function getProductSearchParameters() {
		return array(
			'name'     => Shop_Helper::get_shop_name() ?: Shop_Helper::get_shop_url() . ' ProductSearch',
			'url'      => Shop_Helper::get_controller_url( 'Product_Search', 'run' ),
			'password' => $this->getProductSearchEndpointPassword(),
		);
	}

	/**
	 * Retrieves URL of a controller that will handle webhook calls (GET for verification, POST for handling)
	 *
	 * @return string
	 */
	public function getCrEventHandlerURL() {
		return Shop_Helper::get_event_handler_url();
	}

	/**
	 * Return url of plugin with CleverReach poll popup
	 *
	 * @return string
	 */
	public function getPluginUrl() {
		return get_admin_url( null, 'admin.php?page=cleverreach-wc' );
	}

	/**
	 * Return notification message that will be shown user as system notification
	 *
	 * @return string
	 */
	public function getNotificationMessage() {
		return '';
	}

	/**
	 * Retrieves color code of authentication iframe background.
	 *
	 * @return string Color code.
	 */
	public function getAuthIframeColor() {
		return 'f1f1f1';
	}

	/**
	 * Sets if user is online.
	 *
	 * @param bool $user_online
	 *
	 * @throws \Exception
	 */
	public function setUserOnline( $user_online ) {
		$user_was_online = $this->isUserOnline();

		parent::setUserOnline( $user_online );

		if ( $user_was_online && ! $user_online ) {
			Shop_Helper::push_user_offline_notification();
		}
	}

	/**
	 * Gets default handling of newsletter field
	 *
	 * @return string Possible values 'all', 'none'.
	 */
	public function get_default_newsletter_status() {
		return $this->getConfigRepository()->get( 'CLEVERREACH_NEWSLETTER_STATUS' );
	}

	/**
	 * Sets default handling of newsletter field.
	 *
	 * @param string $newsletter_status Default newsletter subscription status.
	 */
	public function set_default_newsletter_status( $newsletter_status ) {
		$this->getConfigRepository()->set( 'CLEVERREACH_NEWSLETTER_STATUS', $newsletter_status );
	}

	/**
	 * Gets language for sync.
	 *
	 * @return string
	 */
	public function get_language() {
		return $this->getConfigRepository()->get( 'CLEVERREACH_SYNC_LANGUAGE' );
	}

	/**
	 * Sets language for sync.
	 *
	 * @param string $language Language code.
	 */
	public function set_language( $language ) {
		$this->getConfigRepository()->set( 'CLEVERREACH_SYNC_LANGUAGE', $language );
	}

	/**
	 * Gets database version
	 *
	 * @return string
	 */
	public function get_database_version() {
		return $this->getConfigRepository()->get( 'CLEVERREACH_DATABASE_VERSION' );
	}

	/**
	 * Sets database version for migration scripts
	 *
	 * @param string $database_version Database version.
	 */
	public function set_database_version( $database_version ) {
		$this->getConfigRepository()->set( 'CLEVERREACH_DATABASE_VERSION', $database_version );
	}

	/**
	 * Checks if plugin is updated to version 1.2.0
	 *
	 * @return bool
	 */
	public function is_plugin_updated_to_version_1_2_0() {
		return (bool) $this->getConfigRepository()->get( 'IS_PLUGIN_UPDATED_TO_1_2_0' );
	}

	/**
	 * Sets flag that indicates that plugin is updated to version 1.2.0
	 */
	public function set_plugin_updated_to_version_1_2_0() {
		$this->getConfigRepository()->set( 'IS_PLUGIN_UPDATED_TO_1_2_0', 1 );
	}

	/**
	 * Returns flag that indicates that there is admin message
	 *
	 * @return bool
	 */
	public function show_admin_notice() {
		return (bool) $this->getConfigRepository()->get( 'SHOW_ADMIN_NOTICE' );
	}

	/**
	 * Sets flag that indicates that there is admin message
	 *
	 * @param bool $show_notice
	 */
	public function set_show_admin_notice( $show_notice ) {
		$this->getConfigRepository()->set( 'SHOW_ADMIN_NOTICE', $show_notice );
	}

	/**
	 * Returns notification parameters as array
	 *
	 * @return array notification ['description' => $description, 'url' => $url]
	 */
	public function get_admin_notification_data() {
		$json_notification = $this->getConfigRepository()->get( 'ADMIN_NOTIFICATION' );

		return json_decode( $json_notification, true );
	}

	/**
	 * Stores html formatted message that will be shown to admin
	 *
	 * @param Notification $notification
	 */
	public function save_admin_notification_data( Notification $notification ) {
		$json_notification = json_encode( array(
			'title'       => $notification->getName(),
			'description' => $notification->getDescription(),
			'url'         => $notification->getUrl()
		) );

		$this->getConfigRepository()->set( 'ADMIN_NOTIFICATION', $json_notification );
	}

	/**
	 * Returns label for newsletter subscription
	 *
	 * @return string
	 */
	public function get_subscribe_for_newsletter_label() {
		return $this->getConfigRepository()->get( 'CLEVERREACH_SUBSCRIBE_FOR_NEWSLETTER_LABEL' );
	}

	/**
	 * Saves newsletter subscription label
	 *
	 * @param $label
	 */
	public function save_subscribe_for_newsletter_label( $label ) {
		$this->getConfigRepository()->set( 'CLEVERREACH_SUBSCRIBE_FOR_NEWSLETTER_LABEL', $label );
	}

	/**
	 * Returns whether the subscribe for newsletter checkbox is disabled or not
	 *
	 * @return string
	 */
	public function is_newsletter_checkbox_disabled() {
		return $this->getConfigRepository()->get( 'IS_CLEVERREACH_NEWSLETTER_CHECKBOX_DISABLED' );
	}

	/**
	 * Saves whether the subscribe for newsletter checkbox is disabled or not
	 *
	 * @param $is_checkbox_disabled
	 */
	public function save_newsletter_checkbox_disabled( $is_checkbox_disabled ) {
		$this->getConfigRepository()->set( 'IS_CLEVERREACH_NEWSLETTER_CHECKBOX_DISABLED', $is_checkbox_disabled );
	}

	/**
	 * Gets instance on configuration repository service.
	 *
	 * @return ConfigRepositoryInterface
	 *   Instance of configuration service.
	 */
	protected function getConfigRepository() {
		return ServiceRegister::getService( ConfigRepositoryInterface::CLASS_NAME );
	}
}
