<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Components\InfrastructureServices;

use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Interfaces\Required\HttpClient;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Utility\HttpResponse;

/**
 * Class Http_Client_Service
 *
 * @package CleverReach\WooCommerce\Components\InfrastructureServices
 */
class Http_Client_Service extends HttpClient {
	/**
	 * CURL handler.
	 *
	 * @var resource
	 */
	private $curl_session;

	/**
	 * Configuration service.
	 *
	 * @var Config_Service
	 */
	private $config_service;

	/**
	 * Create and send request.
	 *
	 * @param string     $method HTTP method (GET, POST, PUT, DELETE etc.).
	 * @param string     $url Request URL. Full URL where request should be sent.
	 * @param array|null $headers Request headers to send. Key as header name and value as header content.
	 * @param string     $body Request payload. String data to send request payload in JSON format.
	 *
	 * @return HttpResponse Http response object.
	 *
	 * @throws \CleverReach\WooCommerce\IntegrationCore\Infrastructure\Utility\Exceptions\HttpCommunicationException Only in situation when there is no connection, no response, throw this exception.
	 */
	public function sendHttpRequest( $method, $url, $headers = array(), $body = '' ) {
		$this->set_curl_session_and_common_request_parts( $method, $url, $headers, $body );
		$this->set_curl_session_options_for_synchronous_request();

		return $this->execute_and_return_response_for_synchronous_request( $url );
	}

	/**
	 * Create and send request asynchronously.
	 *
	 * @param string     $method HTTP method (GET, POST, PUT, DELETE etc.).
	 * @param string     $url Request URL. Full URL where request should be sent.
	 * @param array|null $headers Request headers to send. Key as header name and value as header content.
	 * @param string     $body Request payload. String data to send request payload in JSON format.
	 *
	 * @return mixed
	 */
	public function sendHttpRequestAsync( $method, $url, $headers = array(), $body = '' ) {
		$this->set_curl_session_and_common_request_parts( $method, $url, $headers, $body );
		$this->set_curl_session_options_for_asynchronous_request();

		return curl_exec( $this->curl_session );
	}

	/**
	 * Sets cURL session and common request parts.
	 *
	 * @param string $method Request method.
	 * @param string $url Request URL.
	 * @param array  $headers Array of request headers.
	 * @param string $body Request body.
	 */
	private function set_curl_session_and_common_request_parts( $method, $url, array $headers, $body ) {
		$this->initialize_curl_session();
		$this->set_curl_session_options_based_on_method( $method );
		$this->set_curl_session_url_headers_and_body( $method, $url, $headers, $body );
		$this->set_common_options_for_curl_session();
	}

	/**
	 * Initializes cURL session.
	 */
	private function initialize_curl_session() {
		$this->curl_session = curl_init();
	}

	/**
	 * Sets cURL session option based on request method.
	 *
	 * @param string $method Request method.
	 */
	private function set_curl_session_options_based_on_method( $method ) {
		if ( 'DELETE' === $method ) {
			curl_setopt( $this->curl_session, CURLOPT_CUSTOMREQUEST, 'DELETE' );
		}

		if ( 'POST' === $method ) {
			curl_setopt( $this->curl_session, CURLOPT_POST, true );
		}

		if ( 'PUT' === $method ) {
			curl_setopt( $this->curl_session, CURLOPT_CUSTOMREQUEST, 'PUT' );
		}
	}

	/**
	 * Sets cURL session URL, headers, and request body.
	 *
	 * @param string $method Request method.
	 * @param string $url Request URL.
	 * @param array  $headers Array of request headers.
	 * @param string $body Request body.
	 */
	private function set_curl_session_url_headers_and_body( $method, $url, array $headers, $body ) {
		curl_setopt( $this->curl_session, CURLOPT_URL, $url );
		curl_setopt( $this->curl_session, CURLOPT_HTTPHEADER, $headers );
		if ( in_array($method, array('PUT', 'POST'), true ) ) {
			curl_setopt( $this->curl_session, CURLOPT_POSTFIELDS, $body );
		}
	}

	/**
	 * Sets common options for cURL session.
	 */
	private function set_common_options_for_curl_session() {
		curl_setopt( $this->curl_session, CURLOPT_RETURNTRANSFER, true );
		curl_setopt( $this->curl_session, CURLOPT_FOLLOWLOCATION, true );
		curl_setopt( $this->curl_session, CURLOPT_SSL_VERIFYPEER, false );
		curl_setopt( $this->curl_session, CURLOPT_SSL_VERIFYHOST, false );
		// Set default user agent, because for some shops if user agent is missing, request will not work.
		curl_setopt(
			$this->curl_session,
			CURLOPT_USERAGENT,
			'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/64.0.3282.186 Safari/537.36'
		);
	}

	/**
	 * Sets cURL session options for synchronous request.
	 */
	private function set_curl_session_options_for_synchronous_request() {
		curl_setopt( $this->curl_session, CURLOPT_HEADER, true );
	}

	/**
	 * Sets cURL session options for asynchronous request.
	 */
	private function set_curl_session_options_for_asynchronous_request() {
		// Always ensure the connection is fresh.
		curl_setopt( $this->curl_session, CURLOPT_FRESH_CONNECT, true );
		// Timeout super fast once connected, so it goes into async.
		curl_setopt( $this->curl_session, CURLOPT_TIMEOUT_MS, $this->get_config_service()->getAsyncProcessRequestTimeout() );
	}

	/**
	 * Executes and returns response for synchronous request.
	 *
	 * @param string $url Request URL.
	 *
	 * @return HttpResponse
	 * @throws \CleverReach\WooCommerce\IntegrationCore\Infrastructure\Utility\Exceptions\HttpCommunicationException HTTP communication exception.
	 */
	private function execute_and_return_response_for_synchronous_request( $url ) {
		$api_response = curl_exec( $this->curl_session );
		$status_code  = curl_getinfo( $this->curl_session, CURLINFO_HTTP_CODE );
		curl_close( $this->curl_session );

		if ( false === $api_response ) {
			throw new \CleverReach\WooCommerce\IntegrationCore\Infrastructure\Utility\Exceptions\HttpCommunicationException(
				'Request ' . $url . ' failed.'
			);
		}

		return new HttpResponse(
			$status_code,
			$this->get_headers_from_curl_response( $api_response ),
			$this->get_body_from_curl_response( $api_response )
		);
	}

	/**
	 * Returns array of headers from cURL response.
	 *
	 * @param string $response Response string.
	 *
	 * @return array
	 */
	private function get_headers_from_curl_response( $response ) {
		$headers                = array();
		$headers_body_delimiter = "\r\n\r\n";
		$header_text            = substr( $response, 0, strpos( $response, $headers_body_delimiter ) );
		$headers_delimiter      = "\r\n";

		foreach ( explode( $headers_delimiter, $header_text ) as $i => $line ) {
			if ( 0 === $i ) {
				$headers[] = $line;
			} else {
				list( $key, $value ) = explode( ': ', $line );
				$headers[ $key ]     = $value;
			}
		}

		return $headers;
	}

	/**
	 * Returns body from cURL response.
	 *
	 * @param string $response Response string.
	 *
	 * @return string
	 */
	private function get_body_from_curl_response( $response ) {
		$headers_body_delimiter        = "\r\n\r\n";
		$body_starting_position_offset = 4; // Number of special signs in delimiter.

		return substr(
			$response,
			strpos( $response, $headers_body_delimiter ) + $body_starting_position_offset,
			strlen( $response )
		);
	}

	/**
	 * Returns configuration service.
	 *
	 * @return Config_Service
	 */
	private function get_config_service() {
		if ( null === $this->config_service ) {
			$this->config_service = ServiceRegister::getService( Configuration::CLASS_NAME );
		}

		return $this->config_service;
	}
}
