<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Components\Repositories;

use CleverReach\WooCommerce\Components\Utility\Database;

/**
 * Class Base_Repository
 *
 * @package CleverReach\WooCommerce\Components\Repositories
 */
class Base_Repository {
	/**
	 * Database session object.
	 *
	 * @var \wpdb
	 */
	protected $db;
	/**
	 * Table name.
	 *
	 * @var string
	 */
	protected $table_name;
	/**
	 * Database handler.
	 *
	 * @var Database
	 */
	protected $database_handler;
	/**
	 * ID column.
	 *
	 * @var string
	 */
	protected $id_column = 'id';

	/**
	 * Base_Repository constructor.
	 */
	public function __construct() {
		global $wpdb;
		$this->db               = $wpdb;
		$this->database_handler = new Database( $this->db );
	}

	/**
	 * Returns full table name, with database prefix.
	 *
	 * @param string $table_name Table name.
	 *
	 * @return string
	 */
	public static function get_table_name( $table_name ) {
		global $wpdb;

		return $wpdb->prefix . $table_name;
	}

	/**
	 * Inserts single record and returns insert ID, if available.
	 *
	 * @param array $item Array of key value pairs.
	 *
	 * @return int
	 */
	public function insert( $item ) {
		$this->db->insert( $this->table_name, $item );

		return (int) $this->db->insert_id;
	}

	/**
	 * Updates single record.
	 *
	 * @param array $item An item array of record as key-value.
	 * @param array $conditions List of simple search filters as key-value pair to find records to update.
	 *
	 * @return false|int
	 *
	 * @throws \InvalidArgumentException Invalid argument.
	 */
	public function update( $item, $conditions ) {
		if ( empty( $conditions ) && empty( $item[ $this->id_column ] ) ) {
			throw new \InvalidArgumentException( 'Condition needs to be set for update' );
		}

		return $this->db->update( $this->table_name, $item, $conditions );
	}

	/**
	 * Deletes a record.
	 *
	 * @param array $conditions Delete conditions.
	 */
	public function delete( $conditions ) {
		$this->db->delete( $this->table_name, $conditions );
	}

	/**
	 * Finds record by primary key.
	 *
	 * @param mixed $id ID of the record.
	 *
	 * @return array|null
	 */
	public function find_by_pk( $id ) {
		return $this->find_one( array( $this->id_column => $id ) );
	}

	/**
	 * Finds one record by provided conditions.
	 *
	 * @param array $filter_by List of simple search filters as key-value pair. Leave empty for unfiltered result.
	 * @param array $sort_by List of sorting options where key is field and value is sort direction
	 *  ("ASC" or "DESC"). Leave empty for default sorting.
	 * @param bool  $lock Lock.
	 *
	 * @return array
	 */
	public function find_one( $filter_by = null, $sort_by = null, $lock = false ) {
		$item = $this->find_all( $filter_by, $sort_by, 0, 1, null, $lock );

		return ! empty( $item ) ? $item[0] : null;
	}

	/**
	 * Finds all records for provided conditions ordered in provided sort.
	 *
	 * @param array $filter_by List of simple search filters as key-value pair. Leave empty for unfiltered result.
	 * @param array $sort_by List of sorting options where key is field and value is sort direction
	 *  ("ASC" or "DESC"). Leave empty for default sorting.
	 * @param int   $start From which record index result set should start.
	 * @param int   $limit Max number of records that should be returned (default is 10).
	 * @param array $select List of table columns to return. Column names could have alias as well.
	 *      If empty, all columns are returned.
	 * @param bool  $lock Lock.
	 *
	 * @return array
	 */
	public function find_all(
		$filter_by = null,
		$sort_by = null,
		$start = 0,
		$limit = 0,
		$select = null,
		$lock = false
	) {
		$sql  = 'SELECT ' . $this->build_select( $select )
		. " FROM `$this->table_name` "
		. $this->build_condition( $filter_by )
		. $this->build_order_by( $sort_by );
		$sql .= ( $limit > 0 ? " LIMIT $start,$limit" : '' );
		if ( $lock ) {
			$sql .= ' FOR UPDATE';
		}

		$result = $this->db->get_results( $sql, ARRAY_A );

		return false !== $result ? $result : null;
	}

	/**
	 * Escapes provided value.
	 *
	 * @param mixed $value Value to be escaped.
	 *
	 * @return string
	 */
	protected function escape( $value ) {
		return addslashes( $value );
	}

	/**
	 * Checks if value exists and escapes it if it's not.
	 *
	 * @param mixed $value Value to be escaped.
	 *
	 * @return string
	 */
	protected function escape_value( $value ) {
		return null === $value ? 'NULL' : "'" . $this->escape( $value ) . "'";
	}

	/**
	 * Escapes an array of values.
	 *
	 * @param array $values Array of values to be escaped.
	 *
	 * @return array
	 */
	protected function escape_values( $values ) {
		$result = array();
		foreach ( $values as $value ) {
			$result[] = $this->escape_value( $value );
		}

		return $result;
	}

	/**
	 * Builds select query.
	 *
	 * @param array $select Array of columns to be included in the result.
	 *
	 * @return string
	 */
	private function build_select( $select ) {
		if ( empty( $select ) ) {
			return '*';
		}

		$result = array();
		foreach ( $select as $field ) {
			$result[] = '`' . $field . '`';
		}

		return implode( ', ', $result );
	}

	/**
	 * Builds ORDER BY part of select query.
	 *
	 * @param array $order_by Array of order columns in the result.
	 *
	 * @return string
	 */
	private function build_order_by( $order_by ) {
		if ( empty( $order_by ) ) {
			return '';
		}

		$sort = array();
		foreach ( $order_by as $key => $order ) {
			$sort[] = "`$key` $order";
		}

		return ' ORDER BY ' . implode( ', ', $sort );
	}

	/**
	 * Builds WHERE part of select query.
	 *
	 * @param array $filter_by Filter conditions in query.
	 *
	 * @return string
	 */
	protected function build_condition( $filter_by ) {
		if ( empty( $filter_by ) ) {
			return '';
		}

		$where = array();
		foreach ( $filter_by as $key => $value ) {
			if ( null === $value ) {
				$where[] = "`$key` IS NULL";
			} else {
				$where[] = "`$key` = '" . $this->escape( $value ) . "'";
			}
		}

		return ' WHERE ' . implode( ' AND ', $where );
	}
}
