<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Components\Repositories;

use CleverReach\WooCommerce\Components\InfrastructureServices\Config_Service;
use CleverReach\WooCommerce\Components\Utility\Shop_Helper;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\OrderItem;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;

/**
 * Class Order_Repository
 *
 * @package CleverReach\WooCommerce\Components\Repositories
 */
class Order_Repository extends Base_Repository {
	const ORDER_POST_TYPE   = 'shop_order';
	const EMAIL_META_KEY    = '_billing_email';
	const CUSTOMER_META_KEY = '_customer_user';

	/**
	 * Orders table.
	 *
	 * @var string
	 */
	protected $orders_table;
	/**
	 * Users table.
	 *
	 * @var string
	 */
	protected $users_table;
	/**
	 * Orders meta table.
	 *
	 * @var string
	 */
	protected $orders_meta_table;
	/**
	 * Array of guest keys.
	 *
	 * @var array
	 */
	private $guest_keys;

	/**
	 * Recipient_Repository constructor.
	 */
	public function __construct() {
		parent::__construct();

		$this->id_column         = 'ID';
		$this->orders_table      = $this->db->posts;
		$this->users_table       = $this->db->users;
		$this->orders_meta_table = $this->db->postmeta;
		$this->guest_keys        = array(
			'first_name',
			'last_name',
			'address_1',
			'address_2',
			'city',
			'state',
			'postcode',
			'country',
			'email',
			'phone',
		);
	}

	/**
	 * Returns order items that belong to provided orders.
	 *
	 * @param array $order_ids Array of order IDs.
	 *
	 * @return OrderItem[]
	 *
	 * @throws \Exception Exception.
	 */
	public function get_order_items( $order_ids ) {
		$order_items = array();
		$site_name   = Shop_Helper::get_shop_name();
		if ( ! empty( $order_ids ) ) {
			foreach ( $order_ids as $id ) {
				/**
				 * WooCommerce order.
				 *
				 * @var \WC_Order $order
				 */
				$order = $this->get_order_by_id( $id );

				if ( null !== $order ) {
					/**
					 * WordPress post.
					 *
					 * @var \WP_Post $post
					 */
					$post = get_post( $id );
					/**
					 * WooCommerce order item.
					 *
					 * @var \WC_Order_Item $item
					 */
					foreach ( $order->get_items() as $item ) {
						$order_items[] = $this->format_order_item( $order, $post, $item, $site_name );
					}
				}
			}
		}

		return $order_items;
	}

	/**
	 * Retrieves order by id.
	 *
	 * @param int $id Order ID.
	 *
	 * @return \WC_Order | null
	 */
	public function get_order_by_id( $id ) {
		$order = wc_get_order( $id );

		return ! empty( $order ) ? $order : null;
	}

	/**
	 * Returns all orders for a customer with a given email.
	 *
	 * @param int $id Customer ID.
	 *
	 * @return array
	 */
	public function get_customer_orders_by_id( $id ) {
		$query = "SELECT pm.post_id
				  FROM $this->orders_meta_table as pm
				  INNER JOIN $this->orders_table as p
				  ON pm.post_id=p.ID
				  AND p.post_type='" . self::ORDER_POST_TYPE . "'
				  WHERE pm.meta_key='" . self::CUSTOMER_META_KEY . "'
				  AND pm.meta_value='$id'
				  ORDER BY p.post_date DESC";

		$result = $this->db->get_col( $query, 0 );

		return ! empty( $result ) ? $result : array();
	}

	/**
	 * Retrieves all guest ids.
	 *
	 * @return array
	 */
	public function get_all_guest_ids() {
		$query = "SELECT ID 
				  FROM $this->orders_table AS o 
				  INNER JOIN " . $this->orders_meta_table . " AS pm
				  ON o.ID=pm.post_id and pm.meta_key='" . self::CUSTOMER_META_KEY . "' AND pm.meta_value=0
				  WHERE o.post_type='" . self::ORDER_POST_TYPE . "'  
					  AND o.ID NOT IN 
			          (
			                SELECT post_id 
			                FROM $this->orders_meta_table AS om 
			                WHERE om.meta_key='" . self::EMAIL_META_KEY . "' 
			                AND om.meta_value IN 
			                (
			                    SELECT user_email 
			                    FROM $this->users_table
			                )
			          )
			          AND o.ID IN
			          (
			                SELECT MAX(post_id)
			                FROM $this->orders_meta_table as om2
			                WHERE om2.meta_key='" . self::EMAIL_META_KEY . "'
			                AND NULLIF(om2.meta_value, '') IS NOT NULL
			                GROUP BY om2.meta_value
			          )";

		$result = $this->db->get_col( $query, 0 );

		return ! empty( $result ) ? $result : array();
	}

	/**
	 * Retrieve guest.
	 *
	 * @param string $id order id containing guest billing information.
	 *
	 * @return array
	 */
	public function get_guest_by_id( $id ) {
		$guest = array();

		$order = wc_get_order( $id );
		if ( empty( $order ) ) {
			return $guest;
		}

		$order_data = $order->get_data();

		$guest['id']         = $id;
		$guest['registered'] = $order->get_date_created();
		$this->format_guest( $guest, $order_data['billing'] );

		return $guest;
	}

	/**
	 * Returns number of orders for guest customer with provided e-mail address.
	 *
	 * @param string $email Guest customer e-mail address.
	 *
	 * @return int Number of guest customer's orders.
	 */
	public function get_guest_order_count( $email ) {
		return count( $this->get_guest_orders( $email ) );
	}

	/**
	 * Returns all orders for guest customer with provided email address.
	 *
	 * @param string $email Guest email address.
	 *
	 * @return array Array of order IDs.
	 */
	public function get_guest_orders( $email ) {
		$this->table_name = $this->orders_meta_table;

		$orders = $this->find_all(
			array(
				'meta_key'   => self::EMAIL_META_KEY,
				'meta_value' => $email,
			),
			array( 'meta_id' => 'DESC' ),
			0,
			0,
			array( 'post_id' )
		);

		if ( null === $orders ) {
			return array();
		}

		$guest_orders = array();
		foreach ( $orders as $order ) {
			$customer_id = (int) get_post_meta( $order['post_id'], self::CUSTOMER_META_KEY, true );
			if ( 0 === $customer_id ) {
				// Order is not assigned to an existing customer.
				$guest_orders[] = $order['post_id'];
			}
		}

		return $guest_orders;
	}

	/**
	 * Returns order ID of the last order made by guest customer with given email address on frontend checkout.
	 *
	 * @param string $email Guest customer email address.
	 *
	 * @return int Order ID if order exists, 0 if it doesn't.
	 */
	public function get_last_guest_checkout_order( $email ) {
		$this->table_name = $this->orders_meta_table;

		$orders = $this->find_all(
			array(
				'meta_key'   => self::EMAIL_META_KEY,
				'meta_value' => $email,
			),
			array( 'post_id' => 'DESC' ),
			0,
			0,
			array( 'post_id' )
		);

		if ( empty( $orders ) ) {
			return 0;
		}

		foreach ( $orders as $order ) {
			if ( 'checkout' === get_post_meta( $order['post_id'], '_created_via', true ) ) {
				return $order['post_id'];
			}
		}

		return 0;
	}

	/**
	 * Formats one order item in proper format for order items sync.
	 *
	 * @param \WC_Order      $order WooCommerce order.
	 * @param \WP_Post       $post WordPress post.
	 * @param \WC_Order_Item $item WooCommerce order item.
	 * @param string         $site_name Name of the site.
	 *
	 * @return OrderItem
	 *
	 * @throws \Exception Exception.
	 */
	private function format_order_item( $order, $post, $item, $site_name ) {
		$product    = wc_get_product( wc_get_order_item_meta( $item->get_id(), '_product_id' ) );
		$order_item = new OrderItem( $item->get_id(), $item->get_name(), $order->get_order_number() );

		/**
		 * WooCommerce product.
		 *
		 * @var \WC_Product $product
		 */
		if ( is_object( $product ) ) {
			$attributes = $product->get_attributes();
			$categories = get_the_terms( $product->get_id(), 'product_cat' );
			$order_item->setProductId( $product->get_id() );
			$order_item->setAttributes( $this->format_product_attributes( $product, $attributes ) );
		}

		$category_names = array();
		if ( ! empty( $categories ) ) {
			$category_names = array_map(
				function ( $category ) {
					return $category->name;
				},
				$categories
			);
		}

		$order_item->setStamp( new \DateTime( $post->post_date ) );
		$order_item->setPrice( (float) wc_get_order_item_meta( $item->get_id(), '_line_total' ) );
		$order_item->setCurrency( get_woocommerce_currency() );
		$order_item->setAmount( (int) wc_get_order_item_meta( $item->get_id(), '_qty' ) );
		$order_item->setProductSource( $this->get_config_service()->getIntegrationName() . ' - ' . $site_name );
		$order_item->setProductCategory( $category_names );
		$order_item->setRecipientEmail( $order->get_billing_email() );
		$mailing_id = $order->get_meta( '_cr_mailing_id' );
		if ( ! empty( $mailing_id ) ) {
			$order_item->setMailingId( $mailing_id );
		}

		return $order_item;
	}

	/**
	 * Returns an array of all attribute option labels and values in format: AttrLabel => AttrValues.
	 * If one attribute has multiple values, they are separated with pipes.
	 *
	 * @param \WC_Product $product WooCommerce product.
	 * @param array       $attributes Array of WC_Product_Attribute objects.
	 *
	 * @return array Array in format: AttrLabel => AttrValues
	 */
	private function format_product_attributes( $product, $attributes ) {
		if ( empty( $attributes ) ) {
			return array();
		}

		$formatted_attributes = array();
		/**
		 * WooCommerce product attribute.
		 *
		 * @var \WC_Product_Attribute $attribute
		 */
		foreach ( $attributes as $key => $value ) {
			$label     = wc_attribute_label( $key );
			$attribute = $attributes[ $key ];

			if ( $attribute['is_taxonomy'] ) {
				$formatted_attributes[ $label ] = implode(
					'|',
					wc_get_product_terms( $product->get_id(), $attribute['name'], array( 'fields' => 'names' ) )
				);
			} else {
				$formatted_attributes[ $label ] = $attribute['value'];
			}
		}

		return $formatted_attributes;
	}

	/**
	 * Formats guest.
	 *
	 * @param array $guest Array of guest data.
	 * @param array $billing_information Guest billing information.
	 */
	private function format_guest( array &$guest, array $billing_information ) {
		foreach ( $this->guest_keys as $key ) {
			$guest[ $key ] = ! empty( $billing_information[ $key ] ) ? $billing_information[ $key ] : null;
		}
	}

	/**
	 * Returns configuration service.
	 *
	 * @return Config_Service
	 */
	private function get_config_service() {
		return ServiceRegister::getService( Configuration::CLASS_NAME );
	}
}
