<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Components\Repositories;

/**
 * Class Recipient_Repository
 *
 * @package CleverReach\WooCommerce\Components\Repositories
 */
class Recipient_Repository extends Base_Repository {
	/**
	 * Gets IDs of all users.
	 *
	 * @return array
	 */
	public function get_all_customer_ids() {
		$users = get_users(
			array(
				'role__not_in' => 'Administrator',
				'fields'       => array( $this->id_column ),
			)
		);

		return array_map(
			function ( $user ) {
				return $user->id;
			},
			$users
		);
	}

	/**
	 * Retrieves customer by email.
	 *
	 * @param string $email customers email.
	 *
	 * @return null | \WP_User
	 */
	public function get_customer_by_email( $email ) {
		$user = get_user_by( 'email', $email );

		return $user ?: null;
	}

	/**
	 * Gets IDs of all guest users.
	 *
	 * @return array Guest IDs.
	 */
	public function get_all_guest_ids() {
		$order_repository = new Order_Repository();

		return $order_repository->get_all_guest_ids();
	}

	/**
	 * Gets user IDs for provided role IDs.
	 *
	 * @param array $role_ids Array of role IDs.
	 *
	 * @return array
	 */
	public function get_user_ids_for_roles( $role_ids ) {
		$users = get_users(
			array(
				'role__in'     => $role_ids,
				'role__not_in' => 'Administrator',
				'fields'       => array( $this->id_column ),
			)
		);

		return array_map(
			function ( $user ) {
				return $user->id;
			},
			$users
		);
	}

	/**
	 * Returns users that don't have newsletter status set on them.
	 *
	 * @return array
	 */
	public function get_users_without_newsletter_status() {
		$newsletter_query = "SELECT DISTINCT user_id 
						FROM `{$this->db->usermeta}`
						WHERE meta_key = '{$this->database_handler->get_newsletter_column()}'";

		$admin_query = "SELECT DISTINCT user_id 
					FROM `{$this->db->usermeta}`
				    WHERE meta_key = 'wp_capabilities'
				    AND meta_value LIKE '%administrator%'";

		$query = "SELECT DISTINCT user_id 
				FROM `{$this->db->usermeta}` 
				WHERE user_id NOT IN ({$newsletter_query} UNION {$admin_query})";

		$users = $this->db->get_results( $query, ARRAY_A );

		if ( empty( $users ) ) {
			return array();
		}

		return array_map(
			function ( $user ) {
				return $user['user_id'];
			},
			$users
		);
	}

	/**
	 * Inserts new records regarding newsletter status into user meta table.
	 *
	 * @param array $user_ids Array of users ids.
	 * @param bool  $newsletter_status Newsletter status value.
	 *
	 * @return bool
	 */
	public function insert_newsletter_values( $user_ids, $newsletter_status ) {
		$record_sets = array();

		if (empty($user_ids)) {
			return true;
		}

		foreach ( $user_ids as $user_id ) {
			$record_sets[] = "({$user_id}, '{$this->database_handler->get_newsletter_column()}', {$newsletter_status})";
		}

		$query = "INSERT INTO {$this->db->usermeta} (`user_id`, `meta_key`, `meta_value`) VALUES " . implode( ',', $record_sets );

		return $this->db->query( $query );
	}

	/**
	 * Update user newsletter field.
	 *
	 * @param int $id User ID.
	 * @param int $value Newsletter subscription value.
	 */
	public function update_user_newsletter_field( $id, $value ) {
		update_user_meta( $id, $this->database_handler->get_newsletter_column(), $value );
	}

	/**
	 * Gets users data with provided ID.
	 *
	 * @param int $user_id id of a user.
	 *
	 * @return \stdClass
	 */
	public function get_user_data_by_id( $user_id ) {
		$user_data = get_userdata( $user_id );

		return ! empty( $user_data ) ? $user_data : null;
	}
}
