<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Components\Utility;

use CleverReach\WooCommerce\Components\Repositories\Base_Repository;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\QueueItem;
use wpdb;

/**
 * Class Database
 *
 * @package CleverReach\WooCommerce\Components\Utility
 */
class Database {
	const CONFIG_TABLE            = 'cleverreach_wc_config';
	const PROCESS_TABLE           = 'cleverreach_wc_process';
	const QUEUE_TABLE             = 'cleverreach_wc_queue';
	const ROLES_TABLE             = 'user_roles';
	const NEWSLETTER_STATUS_FIELD = 'cr_newsletter_status';

	/**
	 * WordPress database session.
	 *
	 * @var wpdb
	 */
	private $db;

	/**
	 * Database constructor.
	 *
	 * @param wpdb $db Database session.
	 */
	public function __construct( $db ) {
		$this->db = $db;
	}

	/**
	 * Returns full newsletter column name.
	 */
	public function get_newsletter_column() {
		return $this->db->prefix . self::NEWSLETTER_STATUS_FIELD;
	}

	/**
	 * Checks if plugin was already installed and initialized.
	 *
	 * @return bool
	 */
	public function plugin_already_initialized() {
		$table_name = $this->db->prefix . self::QUEUE_TABLE;

		return $this->db->get_var( "SHOW TABLES LIKE '" . $table_name . "'" ) === $table_name;
	}

	/**
	 * Executes installation scripts.
	 */
	public function install() {
		$queries = $this->prepare_queries_for_install();
		foreach ( $queries as $query ) {
			$this->db->query( $query );
		}
	}

	/**
	 * Executes uninstallation scripts.
	 */
	public function uninstall() {
		$queries = $this->prepare_queries_for_deleting();
		foreach ( $queries as $query ) {
			$this->db->query( $query );
		}
	}

	/**
	 * Executes update database functions.
	 *
	 * @param Version_File_Reader $version_file_reader Version file reader.
	 *
	 * @return bool
	 */
	public function update( $version_file_reader ) {
		while ( $version_file_reader->has_next() ) {
			$statements = $version_file_reader->read_next();
			foreach ( $statements as $statement ) {
				try {
					$this->db->query( $statement );
				} catch ( \Exception $ex ) {
					Initializer::register_services();
					Logger::logInfo( $ex->getMessage(), 'Database Update' );
					Logger::logInfo( $statement, 'SQL' );

					return false;
				}
			}
		}

		return true;
	}

	/**
	 * Prepares database queries for inserting tables.
	 *
	 * @return array
	 */
	private function prepare_queries_for_install() {
		$queries = array();

		$queries[] = 'CREATE TABLE IF NOT EXISTS `'
		. Base_Repository::get_table_name( self::CONFIG_TABLE ) . '` (
            `id` INT(11) NOT NULL AUTO_INCREMENT,
            `key` VARCHAR(255),
            `value` TEXT,
            PRIMARY KEY (`id`)
        )';

		$queries[] = 'CREATE TABLE IF NOT EXISTS `'
		. Base_Repository::get_table_name( self::PROCESS_TABLE ) . '` (
            `id` VARCHAR(50) NOT NULL,
            `runner` VARCHAR(500) NOT NULL,
            PRIMARY KEY (`id`)
        ) DEFAULT CHARACTER SET utf8';

		$queries[] = 'CREATE TABLE IF NOT EXISTS `'
		. Base_Repository::get_table_name( self::QUEUE_TABLE ) . '` (
            `id` INT(11) NOT NULL AUTO_INCREMENT,
            `status` VARCHAR(30) NOT NULL,
            `priority` INT(11) NOT NULL DEFAULT ' . QueueItem::PRIORITY_MEDIUM . ',
            `type` VARCHAR(100) NOT NULL,
            `queueName` VARCHAR(50) NOT NULL,
            `progress` INT(11) NOT NULL DEFAULT 0,
            `lastExecutionProgress` INT(11) DEFAULT 0,
            `retries` INT(11) NOT NULL DEFAULT 0,
            `failureDescription` VARCHAR(255),
            `serializedTask` LONGTEXT NOT NULL,
            `createTimestamp` INT(11),
            `queueTimestamp` INT(11),
            `lastUpdateTimestamp` INT(11),
            `startTimestamp` INT(11),
            `finishTimestamp` INT(11),
            `failTimestamp` INT(11),
            PRIMARY KEY (`id`)
        ) DEFAULT CHARACTER SET utf8';

		return $queries;
	}

	/**
	 * Preparing database queries for dropping tables.
	 *
	 * @return array
	 */
	private function prepare_queries_for_deleting() {
		$queries = array();

		$queries[] = 'DROP TABLE IF EXISTS ' . Base_Repository::get_table_name( self::CONFIG_TABLE );
		$queries[] = 'DROP TABLE IF EXISTS ' . Base_Repository::get_table_name( self::PROCESS_TABLE );
		$queries[] = 'DROP TABLE IF EXISTS ' . Base_Repository::get_table_name( self::QUEUE_TABLE );

		return $queries;
	}
}
