<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Components\Utility;

use CleverReach\WooCommerce\Components\InfrastructureServices\Config_Service;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\Notification;
use CleverReach\WooCommerce\Components\BusinessLogicServices\Notification_Service;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Interfaces\Notifications;

/**
 * A utility class that utilises data related to user's shop.
 *
 * Class Shop_Helper
 *
 * @package CleverReach\WooCommerce\Components\Utility
 */
class Shop_Helper {
	/**
	 * Database date format
	 */
	const DATE_FORMAT = 'Y-m-d H:i:s';

	/**
	 * Shop URL.
	 *
	 * @var string
	 */
	private static $shop_url;

	/**
	 * Shop name.
	 *
	 * @var string
	 */
	private static $shop_name;

	/**
	 * Returns whether CleverReach plugin is enabled.
	 *
	 * @return bool
	 */
	public static function is_plugin_enabled() {
		if ( self::is_plugin_active_for_network() ) {
			return true;
		}

		return self::is_plugin_active_for_current_site();
	}

	/**
	 * Returns if CleverReach plugin is active through network
	 *
	 * @return bool
	 */
	public static function is_plugin_active_for_network() {
		if ( ! is_multisite() ) {
			return false;
		}

		$plugins = get_site_option( 'active_sitewide_plugins' );

		return isset( $plugins[ self::get_plugin_name() ] );
	}

	/**
	 * Returns if CleverReach plugin is active for current site
	 *
	 * @return bool
	 */
	public static function is_plugin_active_for_current_site() {
		return in_array(
			self::get_plugin_name(),
			apply_filters( 'active_plugins', get_option( 'active_plugins' ) ),
			true
		);
	}

	/**
	 * Returns the name of the plugin
	 *
	 * @return string
	 */
	public static function get_plugin_name() {
		return plugin_basename( dirname( dirname( __DIR__ ) ) . '/cleverreach-wc.php' );
	}

	/**
	 * Checks if WooCommerce is active in the shop.
	 *
	 * @return bool
	 */
	public static function is_woocommerce_active() {
		return self::is_plugin_active( 'woocommerce.php' );
	}

	/**
	 * Checks if cURL library is installed and enabled on the system.
	 *
	 * @return bool
	 */
	public static function is_curl_enabled() {
		return function_exists( 'curl_version' );
	}

	/**
	 * Returns language code for selected language in shop's admin panel. If not set, gets language for default shop.
	 *
	 * @return string
	 */
	/**
	 * Returns language code to be used in all synchronization.
	 *
	 * @return string
	 */
	public static function get_sync_language() {
		/**
		 * Configuration service.
		 *
		 * @var Config_Service $config_service
		 */
		$config_service = ServiceRegister::getService( Configuration::CLASS_NAME );

		return $config_service->get_language();
	}

	/**
	 * Gets interface language for given shop.
	 *
	 * @return string
	 */
	public static function get_shop_language() {
		return get_locale();
	}

	/**
	 * Gets integration name + shop name (shop domain).
	 *
	 * @return string
	 */
	public static function get_name() {
		$config_service = ServiceRegister::getService( Configuration::CLASS_NAME );

		if ( empty( self::$shop_name ) ) {
			self::$shop_name = $config_service->getIntegrationName()
				. ': ' . self::get_shop_name() . ' (' . self::get_shop_url() . ')';
		}

		return self::$shop_name;
	}

	/**
	 * Gets the name of the shop
	 *
	 * @return string
	 */
	public static function get_shop_name() {
		$name = get_bloginfo( 'name' );

		return $name ?: '';
	}

	/**
	 * Gets URL for CleverReach controller.
	 *
	 * @param string $name Name of the controller without "CleverReach" and "Controller".
	 * @param string $action Name of the action.
	 * @param array  $params Associative array of parameters.
	 *
	 * @return string
	 */
	public static function get_controller_url( $name, $action = '', array $params = array() ) {
		$query = array( 'cleverreach_wc_controller' => $name );
		if ( ! empty( $action ) ) {
			$query['action'] = $action;
		}

		$query = array_merge( $query, $params );

		return get_site_url() . '/?' . http_build_query( $query );
	}

	/**
	 * Returns authentication callback URL.
	 *
	 * @return string
	 */
	public static function get_auth_redirect_url() {
		return self::get_controller_url( 'Callback', 'run' );
	}

	/**
	 * Returns authentication callback URL for refreshing auth tokens.
	 *
	 * @return string
	 */
	public static function get_refresh_auth_redirect_url() {
		return self::get_controller_url( 'Callback', 'run', array( 'refresh' => true ) );
	}

	/**
	 * Returns event handler URL.
	 *
	 * @return string
	 */
	public static function get_event_handler_url() {
		return self::get_controller_url( 'Event_Handler', 'handler' );
	}

	/**
	 * Gets base URL of default shop's frontend.
	 *
	 * @return string
	 */
	public static function get_shop_url() {
		if ( empty( self::$shop_url ) ) {
			self::$shop_url = get_site_url();
			self::$shop_url = str_replace( 'https://', '', self::$shop_url );
			self::$shop_url = str_replace( 'http://', '', self::$shop_url );
		}

		return self::$shop_url;
	}

	/**
	 *  Gets URL to requested file. Plugin root is returned if no parameter is passed.
	 *
	 * @param string $path Path to requested file.
	 *
	 * @return string
	 */
	public static function get_clever_reach_base_url( $path = '' ) {
		return plugins_url( $path, dirname( __DIR__ ) );
	}

	/**
	 * Gets admin user registration data for CleverReach.
	 *
	 * @return array
	 */
	public static function get_register_data() {
		$user_data = self::get_admin_user_data( get_current_user_id() );

		return array(
			'email'     => ! empty( $user_data['email'] ) ? $user_data['email'] : '',
			'firstname' => ! empty( $user_data['firstname'] ) ? $user_data['firstname'] : '',
			'lastname'  => ! empty( $user_data['lastname'] ) ? $user_data['lastname'] : '',
			'company'   => ! empty( $user_data['company'] ) ? $user_data['company'] : '',
			'gender'    => '',
			'street'    => ! empty( $user_data['street'] ) ? $user_data['street'] : '',
			'zip'       => ! empty( $user_data['zip'] ) ? $user_data['zip'] : '',
			'city'      => ! empty( $user_data['city'] ) ? $user_data['city'] : '',
			'country'   => ! empty( $user_data['country'] ) ? $user_data['country'] : '',
			'phone'     => ! empty( $user_data['phone'] ) ? $user_data['phone'] : '',
		);
	}

	/**
	 * Gets administrator data for user with the provided ID.
	 *
	 * @param int $user_id User ID.
	 *
	 * @return array
	 */
	private static function get_admin_user_data( $user_id ) {
		$user     = get_user_by( 'id', $user_id );
		$address1 = get_user_meta( $user_id, 'billing_address_1', true );
		$address2 = get_user_meta( $user_id, 'billing_address_2', true );
		$street   = ( ! empty( $address1 ) ? $address1 : '' )
			. ( ! empty( $address2 ) ? ' ' . $address2 : '' );
		// Country code (ex. DE, FR, IT...).
		$country = get_user_meta( $user_id, 'billing_country', true );

		$user_data = array(
			'firstname' => get_user_meta( $user_id, 'first_name', true ),
			'lastname'  => get_user_meta( $user_id, 'last_name', true ),
			'email'     => $user->user_email,
			'company'   => get_user_meta( $user_id, 'billing_company', true ),
			'street'    => $street,
			'zip'       => get_user_meta( $user_id, 'billing_postcode', true ),
			'city'      => get_user_meta( $user_id, 'billing_city', true ),
			'country'   => ! empty( $country ) ? wc()->countries->countries[ $country ] : '',
			'phone'     => get_user_meta( $user_id, 'billing_phone', true ),
		);

		return $user_data;
	}

	/**
	 * Returns plugin current version.
	 *
	 * @return string
	 */
	public static function get_plugin_version() {
		$plugin_data = get_plugin_data( WP_PLUGIN_DIR . '/' . self::get_plugin_name() );

		return $plugin_data['Version'];
	}

	/**
	 * Checks if CleverReach plugin for WooCommerce is already installed.
	 *
	 * @return bool
	 */
	public static function is_cleverreach_plugin_for_wordpress_installed() {
		return self::is_plugin_active( 'cleverreach-wp.php' );
	}

	/**
	 * Pushes notification that the connection to CleverReach has been lost.
	 *
	 * @throws \Exception
	 */
	public static function push_user_offline_notification() {
		/** @var Notification_Service $notification_service */
		$notification_service = ServiceRegister::getService( Notifications::CLASS_NAME );
		/** @var Config_Service $config_service */
		$config_service = ServiceRegister::getService( Configuration::CLASS_NAME );

		$notification = new Notification(
			null,
			__( 'Your connection between WooCommerce and CleverReach was interrupted!', 'cleverreach-wc' )
		);
		$notification->setDate( new \DateTime( '@' . time() ) );
		$notification->setDescription( __( 'We have detected that the connection and synchronization with your CleverReach account has been interrupted.', 'cleverreach-wc' ) );
		$notification->setUrl( $config_service->getPluginUrl() . '&notification_type=user_offline' );

		$notification_service->push( $notification );
	}

	/**
	 * Checks if plugin is active.
	 *
	 * @param string $plugin_name The name of the plugin main entry point file. For example "cleverreach-wp.php".
	 *
	 * @return bool
	 */
	private static function is_plugin_active( $plugin_name ) {
		$all_plugins = get_option( 'active_plugins' );

		if ( is_multisite() ) {
			$all_plugins = array_merge( $all_plugins, array_keys( get_site_option( 'active_sitewide_plugins' ) ) );
		}

		foreach ( $all_plugins as $plugin ) {
			if ( false !== strpos( $plugin, '/' . $plugin_name ) ) {
				return true;
			}
		}

		return false;
	}
}
