<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Components\Utility;

use CleverReach\WooCommerce\Components\InfrastructureServices\Config_Service;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Interfaces\Exposed\TaskRunnerWakeup;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Logger\Logger;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Exceptions\QueueStorageUnavailableException;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Queue;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\QueueItem;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Task;

/**
 * Class Task_Queue
 *
 * @package CleverReach\WooCommerce\Components\Utility
 */
class Task_Queue {
	/**
	 * Enqueues a task to the queue.
	 *
	 * @param Task $task Task to be enqueued.
	 * @param bool $throw_exception If functions should rethrow an arror, set to true.
	 *
	 * @throws QueueStorageUnavailableException Queue storage unavailable.
	 */
	public static function enqueue( Task $task, $throw_exception = false ) {
		try {
			/**
			 * Configuration service.
			 *
			 * @var Config_Service $config_service
			 */
			$config_service = ServiceRegister::getService( Configuration::CLASS_NAME );
			/**
			 * Queue service.
			 *
			 * @var Queue $queue_service
			 */
			$queue_service = ServiceRegister::getService( Queue::CLASS_NAME );

			$initial_sync_task = $queue_service->findLatestByType( 'Initial_Sync_Task' );
			if ( null !== $initial_sync_task ) {
				$priority = QueueItem::PRIORITY_MEDIUM;
				if ( 'RefreshUserInfoTask' === $task->getType() ) {
					$priority = QueueItem::PRIORITY_HIGH;
				}

				$queue_service->enqueue( $config_service->getQueueName(), $task, '', $priority );
			}
		} catch ( QueueStorageUnavailableException $ex ) {
			Logger::logDebug(
				wp_json_encode(
					array(
						'Message'          => 'Failed to enqueue task ' . $task->getType(),
						'ExceptionMessage' => $ex->getMessage(),
						'ExceptionTrace'   => $ex->getTraceAsString(),
						'TaskData'         => serialize( $task ),
					)
				),
				'Integration'
			);

			if ( $throw_exception ) {
				throw $ex;
			}
		}
	}

	/**
	 * Calls the wakeup on task runner.
	 */
	public static function wakeup() {
		if ( Shop_Helper::is_plugin_active_for_current_site() && Shop_Helper::is_curl_enabled() ) {
			/**
			 * Wakeup service.
			 *
			 * @var TaskRunnerWakeup $wakeup_service
			 */
			$wakeup_service = ServiceRegister::getService( TaskRunnerWakeup::CLASS_NAME );
			$wakeup_service->wakeup();
		}
	}
}
