<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Components;

use CleverReach\WooCommerce\Components\Exceptions\Unable_To_Create_Hook_Handler_Exception;
use CleverReach\WooCommerce\Components\HookHandlers\Hook_Handler_Factory;

/**
 * Class Hook_Handler
 *
 * @package CleverReach\WooCommerce\Components
 */
class Hook_Handler {
	/**
	 * Registers hook handlers.
	 *
	 * @throws Unable_To_Create_Hook_Handler_Exception When hookhandler is unknown.
	 */
	public function register_hooks() {
		if ( is_multisite() ) {
			$this->register_hook( 'add_user_to_blog', 'add_user_to_blog', 100, 1 );
			$this->register_hook( 'profile_update', 'profile_update_multisite', 100, 2 );
			$this->register_hook( 'remove_user_from_blog', 'remove_user_from_blog', 100, 1 );
			$this->register_hook( 'set_user_role', 'add_user_role_multisite', 100, 2 );
			$this->register_hook( 'ure_user_permissions_update', 'add_user_role_multisite', 100, 2 );

			if ( $this->should_register_address_updated_hook() ) {
				$this->register_hook(
					'woocommerce_customer_save_address',
					'address_updated_multisite',
					100,
					2
				);
			}
		} else {
			$this->register_hook( 'profile_update', 'profile_update_single_site', 100, 2 );
			$this->register_hook( 'delete_user', 'delete_user', 100, 1 );
			$this->register_hook( 'set_user_role', 'add_user_role_single_site', 100, 2 );
			$this->register_hook( 'ure_user_permissions_update', 'add_user_role_single_site', 100, 2 );

			if ( $this->should_register_address_updated_hook() ) {
				$this->register_hook(
					'woocommerce_customer_save_address',
					'address_updated_single_site',
					100,
					2
				);
			}
		}

		$this->register_hook( 'user_register', 'user_register', 100, 1 );
		$this->register_hook( 'update_option', 'update_option', 100, 3 );
		$this->register_hook( 'woocommerce_thankyou', 'order_created', 100, 1 );
		$this->register_hook( 'woocommerce_process_shop_order_meta', 'order_created', 100, 1 );
		$this->register_hook( 'woocommerce_checkout_update_order_meta', 'get_newsletter_from_order', 100, 1 );
		$this->register_hook( 'woocommerce_before_single_product', 'get_product', 100, 0 );
	}

	/**
	 * Registers hook.
	 *
	 * @param string $name Hook name.
	 *
	 * @param string $handler Hook handler.
	 *
	 * @param int    $priority Hook priority.
	 *
	 * @param int    $number_of_arguments Number of arguments of hook handler method.
	 *
	 * @throws Exceptions\Unable_To_Create_Hook_Handler_Exception When action is unknown.
	 */
	private function register_hook( $name, $handler, $priority, $number_of_arguments ) {
		add_action(
			$name,
			array( Hook_Handler_Factory::create( $handler ), 'handle' ),
			$priority,
			$number_of_arguments
		);
	}

	/**
	 * Checks whether address related hooks should be registered.
	 *
	 * @return bool
	 */
	private function should_register_address_updated_hook() {
		if ( ! is_admin() ) {
			return false;
		}

		if ( ! function_exists( 'get_plugin_data' ) ) {
			require_once ABSPATH . 'wp-admin/includes/plugin.php';
		}

		$data = array();
		foreach ( get_option( 'active_plugins' ) as $plugin ) {
			if ( false !== strpos( $plugin, '/woocommerce.php' ) ) {
				$data = get_plugin_data( WP_PLUGIN_DIR . '/' . $plugin );

				break;
			}
		}

		return ! empty( $data['Version'] ) && version_compare( $data['Version'], '3.2.0', '<' );
	}
}
