<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Controllers;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

use CleverReach\WooCommerce\Components\InfrastructureServices\Config_Service;
use CleverReach\WooCommerce\Components\Sync\Initial_Sync_Task;
use CleverReach\WooCommerce\Components\Utility\Shop_Helper;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\AuthInfo;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Interfaces\Proxy;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Proxy\AuthProxy;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Sync\RefreshUserInfoTask;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Exceptions\BadAuthInfoException;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Exceptions\QueueStorageUnavailableException;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Queue;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\QueueItem;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Utility\Exceptions\HttpRequestException;

/**
 * Class Clever_Reach_Callback_Controller
 *
 * @package CleverReach\WooCommerce\Controllers
 */
class Clever_Reach_Callback_Controller extends Clever_Reach_Base_Controller {

	/**
	 * @var Proxy
	 */
	private $proxy;

	/**
	 * Run the request
	 *
	 * @throws QueueStorageUnavailableException
	 * @throws \CleverReach\WooCommerce\IntegrationCore\Infrastructure\Exceptions\InvalidConfigurationException
	 * @throws \CleverReach\WooCommerce\IntegrationCore\Infrastructure\Utility\Exceptions\HttpCommunicationException
	 * @throws \CleverReach\WooCommerce\IntegrationCore\Infrastructure\Utility\Exceptions\HttpRequestException
	 * @throws \CleverReach\WooCommerce\IntegrationCore\Infrastructure\Utility\Exceptions\RefreshTokenExpiredException
	 * @throws \CleverReach\WordPress\IntegrationCore\Infrastructure\Exceptions\InvalidConfigurationException
	 * @throws \CleverReach\WordPress\IntegrationCore\Infrastructure\Utility\Exceptions\HttpCommunicationException
	 * @throws \CleverReach\WordPress\IntegrationCore\Infrastructure\Utility\Exceptions\HttpRequestException
	 * @throws \CleverReach\WordPress\IntegrationCore\Infrastructure\Utility\Exceptions\RefreshTokenExpiredException
	 */
	public function run() {
		$this->validate_request();

		if ( $this->get_param( 'refresh' ) ) {
			$auth_info = $this->get_auth_info( true );
			$this->refresh_tokens( $auth_info );
		} else {
			$auth_info = $this->get_auth_info( false );
			$this->queue_refresh_user_task( $auth_info );
		}

		include dirname( __DIR__ ) . '/resources/views/close-frame.php';
	}

	/**
	 * Validates the request
	 */
	private function validate_request() {
		$code = $this->get_param( 'code' );

		if ( empty( $code ) ) {
			$this->return_json(
				array(
					'status'  => false,
					'message' => __( 'Wrong parameters. Code not set.', 'clevereach-wc' ),
				)
			);
		}
	}

	/**
	 * Returns auth info.
	 *
	 * @param bool $refresh_tokens Whether this call is made to refresh tokens or not.
	 *
	 * @return mixed
	 *
	 * @throws \CleverReach\WooCommerce\IntegrationCore\Infrastructure\Utility\Exceptions\HttpCommunicationException HTTP communication exception.
	 */
	private function get_auth_info( $refresh_tokens ) {
		$code = $this->get_param( 'code' );

		/**
		 * Authentication proxy service.
		 *
		 * @var AuthProxy $proxy
		 */
		$proxy        = ServiceRegister::getService( AuthProxy::CLASS_NAME );
		$redirect_url = $refresh_tokens ? Shop_Helper::get_refresh_auth_redirect_url() : Shop_Helper::get_auth_redirect_url();
		try {
			/**
			 * User auth info object.
			 *
			 * @var AuthInfo $result
			 */
			$result = $proxy->getAuthInfo( $code, $redirect_url );
		} catch ( BadAuthInfoException $e ) {
			$this->return_json(
				array(
					'status'  => false,
					'message' => $e->getMessage(),
				)
			);
		}

		return $result;
	}

	/**
	 * Refreshes user access tokens.
	 *
	 * @param AuthInfo $auth_info
	 *
	 * @throws QueueStorageUnavailableException
	 * @throws \CleverReach\WordPress\IntegrationCore\Infrastructure\Exceptions\InvalidConfigurationException
	 * @throws \CleverReach\WordPress\IntegrationCore\Infrastructure\Utility\Exceptions\HttpCommunicationException
	 * @throws \CleverReach\WordPress\IntegrationCore\Infrastructure\Utility\Exceptions\HttpRequestException
	 * @throws \CleverReach\WordPress\IntegrationCore\Infrastructure\Utility\Exceptions\RefreshTokenExpiredException
	 */
	private function refresh_tokens( $auth_info ) {
		$this->update_user_info_and_credentials( $auth_info );
	}

	/**
	 * @param AuthInfo $auth_info
	 *
	 * @throws QueueStorageUnavailableException
	 * @throws \CleverReach\WordPress\IntegrationCore\Infrastructure\Exceptions\InvalidConfigurationException
	 * @throws \CleverReach\WordPress\IntegrationCore\Infrastructure\Utility\Exceptions\HttpCommunicationException
	 * @throws \CleverReach\WordPress\IntegrationCore\Infrastructure\Utility\Exceptions\HttpRequestException
	 * @throws \CleverReach\WordPress\IntegrationCore\Infrastructure\Utility\Exceptions\RefreshTokenExpiredException
	 */
	private function update_user_info_and_credentials( AuthInfo $auth_info )
	{
		try {
			/** @var Config_Service $config */
			$config        = ServiceRegister::getService( Configuration::CLASS_NAME );
			$api_user_info = $this->get_user_info( $auth_info );
			$db_user_info  = $config->getUserInfo();
			if ( empty( $db_user_info[ 'id' ] ) ) {
				$config->setUserInfo( $api_user_info );
				$config->setAuthInfo( $auth_info );
				/** @var Queue $queue_service */
				$queue_service = ServiceRegister::getService( Queue::CLASS_NAME );
				$queue_service->enqueue( $config->getQueueName(), new Initial_Sync_Task() );

				return;
			}

			if ( $api_user_info[ 'id' ] === $db_user_info[ 'id' ] ) {
				$config->setUserInfo( $api_user_info );
				$config->setAuthInfo( $auth_info );
			}
		} catch (\Exception $exception) {
			return;
		}
	}

	/**
	 * Returns user info as array
	 *
	 * @param AuthInfo $auth_info
	 *
	 * @return array
	 * @throws HttpRequestException
	 */
	private function get_user_info( AuthInfo $auth_info ) {
		/** @var Proxy $proxy */
		$proxy         = ServiceRegister::getService( Proxy::CLASS_NAME );
		$api_user_info = $proxy->getUserInfo( $auth_info->getAccessToken() );

		if ( empty( $api_user_info ) ) {
			throw new HttpRequestException( 'Failed to fetch user info' );
		}

		return $api_user_info;
	}

	/**
	 * Queues refresh user info task.
	 *
	 * @param AuthInfo $auth_info Auth info object.
	 */
	private function queue_refresh_user_task( $auth_info ) {
		/**
		 * Configuration service.
		 *
		 * @var Config_Service $config_service
		 */
		$config_service = ServiceRegister::getService( Configuration::CLASS_NAME );
		/** @var Queue $queue_service */
		$queue_service  = ServiceRegister::getService( Queue::CLASS_NAME );
		try {
			$config_service->setAuthInfo( $auth_info );
			$queue_service->enqueue(
				$config_service->getQueueName(),
				new RefreshUserInfoTask( $auth_info ),
				'',
				QueueItem::PRIORITY_HIGH
			);
		} catch ( QueueStorageUnavailableException $e ) {
			$this->return_json(
				array(
					'status'  => false,
					'message' => $e->getMessage(),
				)
			);
		}
	}
}
