<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Controllers;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

use CleverReach\WooCommerce\Components\Sync\Initial_Sync_Task;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Queue;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\QueueItem;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Exceptions\QueueItemDeserializationException;

/**
 * Class Clever_Reach_Check_Status_Controller
 *
 * @package CleverReach\WooCommerce\Controllers
 */
class Clever_Reach_Check_Status_Controller extends Clever_Reach_Base_Controller {

	/**
	 * Queue service.
	 *
	 * @var Queue
	 */
	private $queue;

	/**
	 * Gets status of refreshing user info task.
	 */
	public function refresh_user_info() {
		$status = 'finished';

		/**
		 * Queue item.
		 *
		 * @var QueueItem $queue_item
		 */
		$queue_item = $this->get_queue_service()->findLatestByType( 'RefreshUserInfoTask' );
		if ( null !== $queue_item ) {
			$queue_status = $queue_item->getStatus();
			if ( QueueItem::FAILED !== $queue_status && QueueItem::COMPLETED !== $queue_status ) {
				$status = 'in_progress';
			}
		}

		$this->return_json( array( 'status' => $status ) );
	}

	/**
	 * Gets status of initial sync task.
	 *
	 * @throws QueueItemDeserializationException Queue item deserialization exception.
	 */
	public function initial_sync() {
		/**
		 * Initial sync task queue item.
		 *
		 * @var QueueItem $sync_task_queue_item
		 */
		$sync_task_queue_item = $this->get_queue_service()->findLatestByType( 'Initial_Sync_Task' );
		if ( null === $sync_task_queue_item ) {
			$this->return_json( array( 'status' => QueueItem::FAILED ) );
		}

		/**
		 * Initial sync task.
		 *
		 * @var Initial_Sync_Task $sync_task
		 */
		$sync_task     = $sync_task_queue_item->getTask();
		$sync_progress = $sync_task->getProgressByTask();

		$this->return_json(
			array(
				'status'       => $sync_task_queue_item->getStatus(),
				'taskStatuses' => array(
					'subscriber_list' => array(
						'status'   => $this->get_status( $sync_progress['subscriberList'] ),
						'progress' => $sync_progress['subscriberList'],
					),
					'add_fields'      => array(
						'status'   => $this->get_status( $sync_progress['fields'] ),
						'progress' => $sync_progress['fields'],
					),
					'recipient_sync'  => array(
						'status'   => $this->get_status( $sync_progress['recipients'] ),
						'progress' => $sync_progress['recipients'],
					),
				),
			)
		);
	}

	/**
	 * Returns status of the queue item for a given progress.
	 *
	 * @param int $progress Status progress (value between 0 and 100).
	 *
	 * @return string
	 */
	private function get_status( $progress ) {
		$status = QueueItem::QUEUED;
		if ( 0 < $progress && $progress < 100 ) {
			$status = QueueItem::IN_PROGRESS;
		} elseif ( $progress >= 100 ) {
			$status = QueueItem::COMPLETED;
		}

		return $status;
	}

	/**
	 * Returns queue service.
	 *
	 * @return Queue
	 */
	private function get_queue_service() {
		if ( null === $this->queue ) {
			$this->queue = ServiceRegister::getService( Queue::CLASS_NAME );
		}

		return $this->queue;
	}
}
