<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Controllers;

use CleverReach\WooCommerce\Components\BusinessLogicServices\Recipient_Service;
use CleverReach\WooCommerce\Components\InfrastructureServices\Config_Service;
use CleverReach\WooCommerce\Components\Repositories\Recipient_Repository;
use CleverReach\WooCommerce\Components\Utility\Task_Queue;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\Recipient;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Interfaces\Proxy;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Sync\RecipientSyncTask;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Logger\Logger;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Exceptions\QueueStorageUnavailableException;

/**
 * Class Clever_Reach_Event_Handler_Controller
 *
 * @package CleverReach\WooCommerce\Controllers
 */
class Clever_Reach_Event_Handler_Controller extends Clever_Reach_Base_Controller {
	/**
	 * List of allowed events.
	 *
	 * @var array
	 */
	static private $allowed_events = array(
		'receiver.subscribed',
		'receiver.unsubscribed',
	);

	/**
	 * Configuration service.
	 *
	 * @var Config_Service
	 */
	private $config_service;
	/**
	 * Recipients repository.
	 *
	 * @var Recipient_Repository
	 */
	private $recipient_repository;

	/**
	 * Clever_Reach_Event_Handler_Controller constructor.
	 */
	public function __construct() {
		$this->is_internal = false;
	}

	/**
	 * Event handle endpoint.
	 *
	 * @throws QueueStorageUnavailableException When we are unable to enqueue task.
	 */
	public function handler() {
		if ( ! $this->is_post() ) {
			$this->confirm_handler();
		} else {
			$this->handle_event();
		}
	}

	/**
	 * Handles request for validating webhook registration.
	 */
	public function confirm_handler() {
		$secret = $this->get_param( 'secret' );

		if ( null !== $secret ) {
			$verification_token = $this->get_config_service()->getCrEventHandlerVerificationToken();
			status_header( 200 );
			echo esc_html( $verification_token . ' ' . $secret );
		} else {
			status_header( 400 );
		}

		exit();
	}

	/**
	 * Returns configuration service.
	 *
	 * @return Config_Service
	 */
	private function get_config_service() {
		if ( null === $this->config_service ) {
			$this->config_service = ServiceRegister::getService( Configuration::CLASS_NAME );
		}

		return $this->config_service;
	}

	/**
	 * Handles two way synchronization  event.
	 *
	 * @throws QueueStorageUnavailableException When we are unable to enqueue task.
	 */
	private function handle_event() {
		$body         = $this->get_request_body_if_valid();
		$proxy        = ServiceRegister::getService( Proxy::CLASS_NAME );
		$cr_recipient = $proxy->getRecipient( $body['payload']['group_id'], $body['payload']['pool_id'] );

		if ( ! $cr_recipient ) {
			$this->die_with_status( 400 );
		}

		switch ( $body['event'] ) {
			case 'receiver.subscribed':
				$this->handle_user_event( $cr_recipient, true );
				break;
			case 'receiver.unsubscribed':
				$this->handle_user_event( $cr_recipient, false );
				break;
		}

		$this->die_with_status( 200 );
	}

	/**
	 * Retrieves request body.
	 *
	 * @return array
	 */
	private function get_request_body_if_valid() {
		$call_token     = isset( $_SERVER['HTTP_X_CR_CALLTOKEN'] ) ? $_SERVER['HTTP_X_CR_CALLTOKEN'] : null;
		$config_service = $this->get_config_service();
		if ( null === $call_token || $call_token !== $config_service->getCrEventHandlerCallToken() ) {
			$this->die_with_status( 401 );
		}

		$body = json_decode( file_get_contents( 'php://input' ), true );

		$is_event_type_valid = isset( $body['event'] ) && in_array( $body['event'], self::$allowed_events, true );

		$is_event_payload_valid = isset( $body['payload']['group_id'], $body['payload']['pool_id'] );

		if ( ! ( $is_event_type_valid && $is_event_payload_valid ) ) {
			$this->die_with_status( 400 );
		}

		$integrationId = $config_service->getIntegrationId();

		if ( (string)$body['payload']['group_id'] !== (string)$integrationId ) {
			Logger::logInfo( "Group id sent in request body : {$body['payload']['group_id']} !== to our integration id: {$integrationId}");
			$this->die_with_status( 200 );
		}

		return $body;
	}

	/**
	 * Handles recipient subscribed event.
	 *
	 * @param Recipient $cr_recipient subscribed recipient.
	 *
	 * @param bool      $is_subscribed sets recipient subscription status.
	 *
	 * @throws QueueStorageUnavailableException When we are unable to enqueue task.
	 */
	private function handle_user_event( $cr_recipient, $is_subscribed ) {
		$recipient_repository = $this->get_recipient_repository();
		$user                 = $recipient_repository->get_customer_by_email( $cr_recipient->getEmail() );
		if ( null === $user ) {
			return;
		}

		$status = $is_subscribed ? 1 : 0;
		$recipient_repository->update_user_newsletter_field( $user->ID, $status );
		$id = Recipient_Service::CUSTOMER_ID_PREFIX . $user->ID;
		Task_Queue::enqueue( new RecipientSyncTask( array( $id ) ) );
	}

	/**
	 * Retrieves recipient repository.
	 *
	 * @return Recipient_Repository
	 */
	private function get_recipient_repository() {
		if ( null === $this->recipient_repository ) {
			$this->recipient_repository = new Recipient_Repository();
		}

		return $this->recipient_repository;
	}

	/**
	 * Dies with defined status code.
	 *
	 * @param int $status response status code.
	 */
	private function die_with_status( $status ) {
		status_header( $status );
		die();
	}
}
