<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Controllers;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

use CleverReach\WooCommerce\Components\BusinessLogicServices\Recipient_Service;
use CleverReach\WooCommerce\Components\InfrastructureServices\Config_Service;
use CleverReach\WooCommerce\Components\Sync\Initial_Sync_Task;
use CleverReach\WooCommerce\Components\Utility\Shop_Helper;
use CleverReach\WooCommerce\Components\Utility\Task_Queue;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Entity\TagCollection;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Interfaces\Recipients;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Proxy\AuthProxy;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Utility\SingleSignOn\SingleSignOnProvider;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Exceptions\QueueStorageUnavailableException;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Queue;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\QueueItem;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Logger\Logger;

/**
 * Class Clever_Reach_Frontend_Controller
 *
 * @package CleverReach\WooCommerce\Controllers
 */
class Clever_Reach_Frontend_Controller extends Clever_Reach_Base_Controller {

	const CURL_NOT_ENABLED_STATE_CODE      = 'curl-not-enabled';
	const WC_NOT_ACTIVE_STATE_CODE         = 'wc-not-active';
	const WELCOME_STATE_CODE               = 'welcome';
	const INITIAL_SYNC_STATE_CODE          = 'initial-sync';
	const INITIAL_SYNC_SETTINGS_STATE_CODE = 'initial-sync-settings';
	const TOKEN_EXPIRED_STATE_CODE         = 'token-expired';
	const DASHBOARD_STATE_CODE             = 'dashboard';

	const CLEVERREACH_HELP_URL        = 'https://support.cleverreach.de/hc/en-us';
	const CLEVERREACH_DASHBOARD_URL   = '/admin';
	const CLEVERREACH_GDPR_URL        = 'https://www.cleverreach.com/en/features/privacy-security/eu-general-data-protection-regulation-gdpr/';
	const SCRIPT_VERSION              =  6;
	/**
	 * Configuration service.
	 *
	 * @var Config_Service
	 */
	private $config_service;
	/**
	 * Queue service.
	 *
	 * @var Queue
	 */
	private $queue_service;
	/**
	 * AuthProxy service.
	 *
	 * @var AuthProxy
	 */
	private $auth_proxy;
	/**
	 * Page name.
	 *
	 * @var string
	 */
	private $page;

	/**
	 * Clever_Reach_Frontend_Controller constructor.
	 */
	public function __construct() {
		$this->load_global_css();
	}

	/**
	 * Renders appropriate view
	 */
	public function render() {
		$this->set_page();
		$this->load_css();
		$this->load_js();

		Task_Queue::wakeup();

		if ( self::WELCOME_STATE_CODE !== $this->page ) {
			include dirname( __DIR__ ) . '/resources/views/wrapper-start.php';
		}

		include dirname( __DIR__ ) . '/resources/views/' . $this->page . '.php';
		include dirname( __DIR__ ) . '/resources/views/wrapper-end.php';
	}

	/**
	 * Set current page code
	 */
	private function set_page() {
		if ( ! Shop_Helper::is_curl_enabled() ) {
			$this->page = self::CURL_NOT_ENABLED_STATE_CODE;
		} elseif ( ! Shop_Helper::is_woocommerce_active() ) {
			$this->page = self::WC_NOT_ACTIVE_STATE_CODE;
		} elseif ( ! $this->is_auth_token_valid() ) {
			$this->page = self::WELCOME_STATE_CODE;
		} elseif ( ! $this->check_if_refresh_token_exists() || ! $this->is_token_alive() ) {
			$this->page = self::TOKEN_EXPIRED_STATE_CODE;
		} elseif ( ! $this->is_initial_sync_config_set() ) {
			$this->page = self::INITIAL_SYNC_SETTINGS_STATE_CODE;
		} elseif ( $this->is_initial_sync_in_progress( true ) ) {
			$this->page = self::INITIAL_SYNC_STATE_CODE;
		} else {
			$this->page = self::DASHBOARD_STATE_CODE;
		}

		if ( ! empty( $_REQUEST['notification_type'] ) && ! in_array( $this->page, array(
				self::INITIAL_SYNC_SETTINGS_STATE_CODE,
				self::INITIAL_SYNC_STATE_CODE
			), true ) ) {
			$this->get_config_service()->set_show_admin_notice( false );
		}
	}

	/**
	 * Checks is auth token valid.
	 *
	 * @return bool
	 */
	private function is_auth_token_valid() {
		$access_token = $this->get_config_service()->getAccessToken();

		return ! empty( $access_token );
	}

	/**
	 * Returns configuration service.
	 *
	 * @return Config_Service
	 */
	private function get_config_service() {
		if ( null === $this->config_service ) {
			$this->config_service = ServiceRegister::getService( Configuration::CLASS_NAME );
		}

		return $this->config_service;
	}

	/**
	 * Checks if initial sync configuration is set.
	 *
	 * @return bool
	 */
	private function is_initial_sync_config_set() {
		$default_newsletter_status = $this->get_config_service()->get_default_newsletter_status();

		return null !== $default_newsletter_status;
	}

	/**
	 * Checks if there is an initial sync task that exists in task queue. If that task is not there at all,
	 * that means that initial sync never started and it should enqueue initial sync task. If there is
	 * an initial sync task in task queue and its status is either completed or failed, that means that
	 * it is not currently in progress. In any other case it is in progress.
	 *
	 * @param bool $enqueue_task If set to true and there is no initial sync task in the queue, the method will
	 *                          enqueue initial sync task.
	 *
	 * @return bool
	 */
	private function is_initial_sync_in_progress( $enqueue_task = false ) {
		/**
		 * Initial sync task item.
		 *
		 * @var QueueItem $initial_sync_task_item
		 */
		$initial_sync_task_item = $this->get_queue_service()->findLatestByType( 'Initial_Sync_Task' );
		if ( ! $initial_sync_task_item && $enqueue_task ) {
			try {
				$this->get_queue_service()->enqueue(
					$this->get_config_service()->getQueueName(),
					new Initial_Sync_Task()
				);
			} catch ( QueueStorageUnavailableException $e ) {
				// If task enqueue fails do nothing but report that initial sync is in progress.
			}

			return true;
		}

		return $initial_sync_task_item->getStatus() !== QueueItem::COMPLETED
		       && $initial_sync_task_item->getStatus() !== QueueItem::FAILED;
	}

	/**
	 * Returns queue service.
	 *
	 * @return Queue
	 */
	private function get_queue_service() {
		if ( null === $this->queue_service ) {
			$this->queue_service = ServiceRegister::getService( Queue::CLASS_NAME );
		}

		return $this->queue_service;
	}

	/**
	 * Checks if refresh token exists in the database.
	 *
	 * @return bool
	 */
	private function check_if_refresh_token_exists() {
		return null !== $this->get_config_service()->getRefreshToken();
	}

	/**
	 * Returns whether access token is alive.
	 *
	 * @return bool
	 */
	private function is_token_alive() {
		return $this->get_auth_proxy()->isConnected();
	}

	/**
	 * Loads custom CleverReach CSS.
	 */
	private function load_css() {
		$base_url = Shop_Helper::get_clever_reach_base_url( '/resources/' );
		wp_enqueue_style(
			'cleverreach-global-admin-icofont',
			$base_url . 'css/cleverreach-icofont.css',
			array(),
			self::SCRIPT_VERSION
		);
		wp_enqueue_style(
			'font-awesome',
			'https://use.fontawesome.com/releases/v5.5.0/css/all.css',
			array(),
			self::SCRIPT_VERSION
		);

		if ( in_array( $this->page, array( self::WELCOME_STATE_CODE, self::TOKEN_EXPIRED_STATE_CODE ), true ) ) {
			wp_enqueue_style(
				'cleverreach-auth-iframe-style',
				$base_url . 'css/cleverreach-auth-iframe.css',
				array(),
				self::SCRIPT_VERSION
			);
		}
	}

	/**
	 * Loads JS that is used outside CleverReach pages.
	 */
	private function load_global_css() {
		$base_url = Shop_Helper::get_clever_reach_base_url( '/resources/' );

		wp_enqueue_style(
			'cleverreach-global-admin-styles',
			$base_url . 'css/cleverreach.css',
			array(),
			self::SCRIPT_VERSION
		);
	}

	/**
	 * Loads JS scipt for the current page.
	 */
	private function load_js() {
		$base_url = Shop_Helper::get_clever_reach_base_url( '/resources/' );

		wp_enqueue_script(
			'cr_ajax',
			esc_url( $base_url . 'js/cleverreach.ajax.js' ),
			array(),
			self::SCRIPT_VERSION,
			true
		);
		wp_enqueue_script(
			'cr_authorization',
			esc_url( $base_url . 'js/cleverreach.authorization.js' ),
			array(),
			self::SCRIPT_VERSION,
			true
		);
		wp_enqueue_script(
			'cr_status_checker',
			esc_url( $base_url . 'js/cleverreach.status-checker.js' ),
			array(),
			self::SCRIPT_VERSION,
			true
		);

		if ( self::WELCOME_STATE_CODE !== $this->page ) {
			wp_enqueue_script(
				"cr_$this->page",
				esc_url( $base_url . "js/cleverreach.$this->page.js" ),
				array(),
				self::SCRIPT_VERSION,
				true
			);
		}
	}

	/**
	 * Marks that first email is built
	 */
	public function build_first_email() {
		$this->get_config_service()->setIsFirstEmailBuilt( '1' );
		$this->return_json( array( 'status' => 'success' ) );
	}

	/**
	 * Retry initial synchronization
	 */
	public function retry_sync() {
		try {
			$this->get_queue_service()->enqueue(
				$this->get_config_service()->getQueueName(),
				new Initial_Sync_Task()
			);
		} catch ( QueueStorageUnavailableException $e ) {
			Logger::logError( 'Error restarting sync: ' . $e->getMessage() );
			$this->return_json( array( 'status' => 'failed' ) );
		}

		$this->return_json( array( 'status' => 'success' ) );
	}

	/**
	 * Saves initial sync settings.
	 */
	public function set_initial_sync_settings() {
		$newsletter_status = $this->get_param( 'defaultNewsletterStatus' );
		$this->get_config_service()->set_default_newsletter_status( 'true' === $newsletter_status ? 1 : 0 );

		$this->return_json( array( 'status' => 'success' ) );
	}

	/**
	 * Gets configuration for welcome page.
	 *
	 * @return array
	 */
	public function get_welcome_config() {
		$redirect_url  = Shop_Helper::get_auth_redirect_url();
		$register_data = base64_encode( wp_json_encode( Shop_Helper::get_register_data() ) );
		$language      = explode( '_', get_user_locale() );

		return array(
			'helloUrl'       => Shop_Helper::get_clever_reach_base_url(
				'/resources/images/icon_' . __( 'hello', 'cleverreach-wc' ) . '.png'
			),
			'checkStatusUrl' => Shop_Helper::get_controller_url( 'Check_Status', 'refresh_user_info' ),
			'authUrl'        => $this->get_auth_proxy()->getAuthUrl(
				$redirect_url,
				$register_data,
				array( 'lang' => $language[0] )
			),
		);
	}

	/**
	 * Gets configuration for dashboard page.
	 *
	 * @return array
	 */
	public function get_dashboard_config() {
		$user_info                   = $this->get_config_service()->getUserInfo();
		$failure_parameters          = $this->get_initial_sync_failure_parameters();
		$import_statistics_displayed = $this->get_config_service()->isImportStatisticsDisplayed();
		$tags                        = new TagCollection();

		if ( ! $import_statistics_displayed ) {
			/**
			 * Recipient service.
			 *
			 * @var Recipient_Service $recipient_service
			 */
			$recipient_service = ServiceRegister::getService( Recipients::CLASS_NAME );
			$tags              = $recipient_service->getAllTags();

			$this->get_config_service()->setImportStatisticsDisplayed( true );
		}

		return array(
			'logoUrl'                       => Shop_Helper::get_clever_reach_base_url(
				'/resources/images/icon_quickstartmailing.svg'
			),
			'buildFirstEmailUrl'            => Shop_Helper::get_controller_url( 'Frontend', 'build_first_email' ),
			'retrySyncUrl'                  => Shop_Helper::get_controller_url( 'Frontend', 'retry_sync' ),
			'importStatisticsDisplayed'     => $import_statistics_displayed,
			'numberOfSyncedRecipients'      => ! $import_statistics_displayed ?
				$this->get_config_service()->getNumberOfSyncedRecipients() : 0,
			'tags'                          => $tags,
			'recipientId'                   => $user_info['id'],
			'helpUrl'                       => self::CLEVERREACH_HELP_URL,
			'gdprUrl'                       => self::CLEVERREACH_GDPR_URL,
			'buildEmailUrl'                 => SingleSignOnProvider::getUrl(self::CLEVERREACH_DASHBOARD_URL),
			'isFirstEmailBuilt'             => $this->get_config_service()->isFirstEmailBuilt(),
			'integrationName'               => $this->get_config_service()->getIntegrationName(),
			'integrationListName'           => $this->get_config_service()->getIntegrationListName(),
			'isInitialSyncTaskFailed'       => $failure_parameters['isFailed'],
			'initialSyncTaskFailureMessage' => $failure_parameters['description'],
			'getSettingsUrl'                => Shop_Helper::get_controller_url( 'Settings', 'get_settings' ),
			'saveSettingsUrl'               => Shop_Helper::get_controller_url( 'Settings', 'save_settings' )
		);
	}

	/**
	 * Wraps tags into segments and prepares them for rendering.
	 *
	 * @param TagCollection $tags All tags in the system.
	 *
	 * @return string HTML prepared segment list.
	 */
	public static function build_segments( $tags ) {
		$segment_list = '';
		foreach ( $tags as $index => $tag ) {
			if ( 3 === $index ) {
				$segment_list .= '<div class="value">...</div>';
				break;
			}

			$first         = ( $index + 1 ) . ') ' . substr( $tag->getTitle(), 0, - 3 );
			$last          = substr( $tag->getTitle(), strlen( $tag->getTitle() ) - 3 );
			$segment_list .= '<div class="value" title="' . $tag->getTitle() . '">'
							. '<span class="cr-first-part">' . $first . '</span>'
							. '<span class="cr-last-part">' . $last . '</span>'
							. '</div>';
		}

		return $segment_list;
	}

	/**
	 * Gets fail parameters for initial sync task if it failed.
	 *
	 * @return array
	 */
	private function get_initial_sync_failure_parameters() {
		$params = array(
			'isFailed'    => false,
			'description' => '',
		);
		/**
		 * Initial sync task queue item.
		 *
		 * @var QueueItem $initial_sync_task
		 */
		$initial_sync_task = $this->get_queue_service()->findLatestByType( 'Initial_Sync_Task' );
		if ( $initial_sync_task && QueueItem::FAILED === $initial_sync_task->getStatus() ) {
			$params = array(
				'isFailed'    => true,
				'description' => $initial_sync_task->getFailureDescription(),
			);
		}

		return $params;
	}

	/**
	 * Gets configuration for token expired page.
	 *
	 * @return array
	 */
	public function get_token_expired_config() {
		$redirect_url  = Shop_Helper::get_refresh_auth_redirect_url();
		$register_data = base64_encode( wp_json_encode( Shop_Helper::get_register_data() ) );
		$user_info     = $this->get_config_service()->getUserInfo();
		$language      = explode( '_', get_user_locale() );
		$message = '';
		$refresh_token = $this->get_config_service()->getRefreshToken();
		if ( ! empty( $refresh_token ) ) {
			$connection_status_response = $this->get_auth_proxy()->checkConnectionStatus();
			$message                    = $connection_status_response->getMessage();
		}

		return array(
			'logoUrl'        => Shop_Helper::get_clever_reach_base_url( '/resources/images/icon_quickstartmailing.svg' ),
			'checkStatusUrl' => Shop_Helper::get_controller_url( 'Check_Status', 'refresh_user_info' ),
			'authUrl'        => $this->get_auth_proxy()->getAuthUrl(
					$redirect_url,
					$register_data,
					array( 'lang' => $language[0] )
				) . '#login',
			'clientId'       => $user_info['id'],
			'apiMessage'     => $message,
		);
	}

	/**
	 * Gets configuration for initial sync settings page.
	 *
	 * @return array
	 */
	public function get_initial_sync_settings_config() {
		$user_info = $this->get_config_service()->getUserInfo();

		return array(
			'helloUrl'          => Shop_Helper::get_clever_reach_base_url(
				'/resources/images/icon_' . __( 'hello', 'cleverreach-wc' ) . '.png'
			),
			'helpUrl'           => self::CLEVERREACH_HELP_URL,
			'recipientId'       => $user_info['id'],
			'configurationUrl'  => Shop_Helper::get_controller_url( 'Frontend', 'set_initial_sync_settings' ),
			'hasSubscriberRole' => null !== get_role( 'subscriber' ),
		);
	}

	/**
	 * Gets configuration for initial sync page.
	 *
	 * @return array
	 */
	public function get_initial_sync_config() {
		return array(
			'statusCheckUrl'  => Shop_Helper::get_controller_url( 'Check_Status', 'initial_sync' ),
			'integrationName' => $this->get_config_service()->getIntegrationName(),
			'crDashboardUrl'  => SingleSignOnProvider::getUrl( self::CLEVERREACH_DASHBOARD_URL ),
		);
	}

	/**
	 * Returns auth proxy service.
	 *
	 * @return AuthProxy
	 */
	private function get_auth_proxy() {
		if ( null === $this->auth_proxy ) {
			$this->auth_proxy = ServiceRegister::getService( AuthProxy::CLASS_NAME );
		}

		return $this->auth_proxy;
	}
}
