<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Controllers;

use CleverReach\WooCommerce\Components\Repositories\Product_Repository;

/**
 * Class Clever_Reach_Product_Search_Controller
 *
 * @package CleverReach\WooCommerce\Controllers
 */
class Clever_Reach_Product_Search_Controller extends Clever_Reach_Base_Controller {

	/**
	 * Clever_Reach_Product_Search_Controller constructor.
	 */
	public function __construct() {
		$this->is_internal = false;
	}

	/**
	 * Public endpoint for product search.
	 */
	public function run() {
		$action = $this->get_param( 'get' );

		$this->validate_request( $action );

		if ( 'filter' === $action ) {
			$response = $this->get_filters();
		} else {
			$response = $this->search();
		}

		$this->return_json( $response );
	}

	/**
	 * Checks if all parameters are set as required.
	 *
	 * @param string $action Requested action.
	 */
	private function validate_request( $action ) {
		$search_term = $this->get_param( 'searchTerm' );

		if ( null === $action
			|| ( 'search' === $action && empty( $search_term ) )
			|| ! $this->is_post()
			|| ! in_array( $action, array( 'filter', 'search' ), true )
		) {
			status_header( 404 );

			exit();
		}
	}

	/**
	 * Returns search filters, used to search for data.
	 *
	 * @return array
	 */
	private function get_filters() {
		return array(
			array(
				'name'        => __( 'Product name or ID', 'cleverreach-wc' ),
				'description' => __( 'Search by product name or ID', 'cleverreach-wc' ),
				'required'    => true,
				'query_key'   => 'searchTerm',
				'type'        => 'input',
			),
		);
	}

	/**
	 * Performs a search using search term provided in the request.
	 *
	 * @return array
	 */
	private function search() {
		$search_term = $this->get_param( 'searchTerm' );

		return array(
			'settings' => array(
				'type'                => 'product',
				'link_editable'       => false,
				'link_text_editable'  => true,
				'image_size_editable' => true,
			),
			'items'    => $this->get_products_by_search_term_or_id( $search_term ),
		);
	}

	/**
	 * Returns all the products whose title or ID match the provided search term.
	 *
	 * @param string $search_term
	 *
	 * @return array
	 */
	private function get_products_by_search_term_or_id( $search_term ) {
		$product_repository = new Product_Repository();

		$product_ids   = $product_repository->get_product_ids_by_search_term( $search_term );
		$product_ids[] = $search_term;

		return $this->format_products( $product_ids );
	}

	/**
	 * Retrieves products by their IDs and prepares them in appropriate format for the response.
	 *
	 * @param array $product_ids Array of product IDs.
	 *
	 * @return array
	 */
	private function format_products( $product_ids ) {
		$results = array();
		foreach ( $product_ids as $product_id ) {
			$product = wc_get_product( $product_id );
			/**
			 * WordPress post.
			 *
			 * @var \WP_Post $post
			 */
			$post = get_post( $product_id );

			if ( $product ) {
				$results[] = array(
					'title'       => $post->post_title,
					'description' => wp_strip_all_tags( $post->post_content ),
					'content'     => $post->post_content,
					'image'       => get_the_post_thumbnail_url( $post->ID ),
					'url'         => get_permalink( $post->ID ),
					'price'       => get_woocommerce_currency()
									. ' '
									. number_format_i18n( (float) $product->get_price(), 2 ),
				);
			}
		}

		return $results;
	}
}
