<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce;

use CleverReach\WooCommerce\Components\Hook_Handler;
use CleverReach\WooCommerce\Components\InfrastructureServices\Config_Service;
use CleverReach\WooCommerce\Components\Utility\Database;
use CleverReach\WooCommerce\Components\Utility\Initializer;
use CleverReach\WooCommerce\Components\Utility\Shop_Helper;
use CleverReach\WooCommerce\Components\Utility\Task_Queue;
use CleverReach\WooCommerce\Components\Utility\Update\Update_Schema;
use CleverReach\WooCommerce\Components\Utility\Version_File_Reader;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Interfaces\Proxy;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Proxy as ConcreteProxy;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Logger\Logger;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Exceptions\TaskRunnerStatusStorageUnavailableException;
use CleverReach\WooCommerce\Controllers\Clever_Reach_Frontend_Controller;
use CleverReach\WooCommerce\Controllers\Clever_Reach_Index;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Proxy\AuthProxy;
use wpdb;

/**
 * Class Plugin
 *
 * @package CleverReach\WooCommerce
 */
class Plugin {

	const UNCHECKED = 'unchecked';
	const CHECKED = 'checked';
	const UPDATE_OPTION_CHECKED_KEY = 'cleverreach-wc-update-checked';

	/**
	 * WordPress database session.
	 *
	 * @var wpdb
	 */
	public $db;
	/**
	 * Plugin instance.
	 *
	 * @var Plugin
	 */
	protected static $instance;
	/**
	 * Plugin file.
	 *
	 * @var string
	 */
	private $cleverreach_plugin_file;
	/**
	 * Base URL.
	 *
	 * @var string
	 */
	private $base_url;
	/**
	 * Proxy service
	 *
	 * @var ConcreteProxy
	 */
	private $proxy;
	/**
	 * Configuration service.
	 *
	 * @var Config_Service
	 */
	private $config_service;

	/**
	 * Plugin constructor.
	 *
	 * @param wpdb   $wpdb WordPress database session.
	 * @param string $cleverreach_plugin_file Plugin file.
	 */
	public function __construct( $wpdb, $cleverreach_plugin_file ) {
		$this->db                      = $wpdb;
		$this->cleverreach_plugin_file = $cleverreach_plugin_file;
		$this->base_url                = Shop_Helper::get_clever_reach_base_url( '/resources/' );
	}

	/**
	 * Returns singleton instance of the plugin.
	 *
	 * @param wpdb   $wpdb WordPress database session.
	 * @param string $cleverreach_plugin_file Plugin file.
	 *
	 * @return Plugin
	 */
	public static function instance( $wpdb, $cleverreach_plugin_file ) {
		if ( null === self::$instance ) {
			self::$instance = new self( $wpdb, $cleverreach_plugin_file );
		}

		self::$instance->initialize();

		return self::$instance;
	}

	/**
	 * Plugin activation function.
	 *
	 * @param bool $is_network_wide Is plugin network wide.
	 */
	public function activate( $is_network_wide ) {
		if ( ! Shop_Helper::is_curl_enabled() ) {
			deactivate_plugins( plugin_basename( __FILE__ ) );
			wp_die(
				esc_html(
					__(
						'cURL is not installed or enabled in your PHP installation. This is required for background task to work. Please install it and then refresh this page.',
						'cleverreach-wc'
					)
				),
				'Plugin dependency check',
				array( 'back_link' => true )
			);
		}

		if ( ! Shop_Helper::is_woocommerce_active() ) {
			deactivate_plugins( plugin_basename( __FILE__ ) );
			wp_die(
				esc_html( __( 'Please install and activate WooCommerce.', 'cleverreach-wc' ) ),
				'Plugin dependency check',
				array( 'back_link' => true )
			);
		}

		if ( $this->plugin_already_initialized() ) {
			Task_Queue::wakeup();
		} elseif ( $is_network_wide && is_multisite() ) {
			$sites = get_sites();
			foreach ( $sites as $site ) {
				switch_to_blog( $site->blog_id );
				$this->init_database();
				$this->init_config();
				restore_current_blog();
			}
		} else {
			$this->init_database();
			$this->init_config();
		}
	}

	/**
	 * Plugin uninstall method.
	 */
	public function uninstall() {
		delete_site_option( self::UPDATE_OPTION_CHECKED_KEY);
		if ( is_multisite() ) {
			$sites = get_sites();
			foreach ( $sites as $site ) {
				$this->switch_to_site_and_uninstall_plugin( $site->blog_id );
			}
		} else {
			$this->uninstall_plugin_from_site();
		}
	}

	/**
	 * Plugin update method
	 */
	private function update_plugin_if_version_changed() {
		if ( is_multisite() ) {
			$site_ids = get_sites();
			foreach ( $site_ids as $site_id ) {
				switch_to_blog( $site_id->blog_id );
				$this->update_plugin_on_single_site();
				restore_current_blog();
			}
		} else {
			$this->update_plugin_on_single_site();
		}
	}

	/**
	 * Adds cleverreach query variable.
	 *
	 * @param array $vars Filter variables.
	 *
	 * @return array Filter variables.
	 */
	public function plugin_add_trigger( $vars ) {
		$vars[] = 'cleverreach_wc_controller';
		return $vars;
	}

	/**
	 * Trigger action on calling plugin controller.
	 */
	public function plugin_trigger_check() {
		$controller_name = get_query_var( 'cleverreach_wc_controller' );
		if ( ! empty( $controller_name ) ) {
			$controller = new Clever_Reach_Index();
			$controller->index();
		}
	}

	/**
	 * Adds newsletter field to the form.
	 */
	public function register_form_field_newsletter() {
		include plugin_dir_path( $this->cleverreach_plugin_file ) . 'resources/views/register-form-newsletter.php';
	}

	/**
	 * Adds newsletter field to the form.
	 */
	public function register_checkout_form_newsletter() {
		include plugin_dir_path( $this->cleverreach_plugin_file ) . 'resources/views/checkout-form-newsletter.php';
	}

	/**
	 * Creates CleverReach item in administrator menu.
	 */
	public function create_admin_menu() {
		$controller = new Clever_Reach_Frontend_Controller();
		add_submenu_page(
			'woocommerce',
			'CleverReach®',
			'CleverReach®',
			'manage_options',
			'cleverreach-wc',
			array( $controller, 'render' )
		);
	}

	/**
	 * Initializes base CleverReach tables and values if plugin is accessed from a new site.
	 */
	public function initialize_new_site() {
		$db = new Database( $this->db );
		if ( ! $db->plugin_already_initialized() ) {
			$this->init_database();
			$this->init_config();
		}

		$is_checked = get_site_option( self::UPDATE_OPTION_CHECKED_KEY );
		if ( $is_checked !== self::CHECKED ) {
			$this->update_plugin_if_version_changed();
			update_site_option( self::UPDATE_OPTION_CHECKED_KEY, self::CHECKED );
		}
	}

	/**
	 * Starts WordPress session.
	 */
	public function start_session() {
		if ( ! session_id() ) {
			session_start( array( 'read_and_close' => true ) );
		}
	}

	/**
	 * Ends WordPress session.
	 */
	public function end_session() {
		if ( PHP_SESSION_ACTIVE === session_status() ) {
			session_destroy();
		}
	}

	/**
	 * Loads plugin translations.
	 */
	public function load_plugin_text_domain() {
		unload_textdomain( 'cleverreach-wc' );
		load_plugin_textdomain(
			'cleverreach-wc',
			false,
			plugin_basename( dirname( $this->cleverreach_plugin_file ) ) . '/i18n/languages'
		);
	}

	/**
	 * Hook that triggers when network site is deleted and removes plugin data related to that site from the network.
	 *
	 * @param int $site_id Site identifier.
	 */
	public function uninstall_plugin_from_deleted_site( $site_id ) {
		$this->switch_to_site_and_uninstall_plugin( $site_id );
	}

	/**
	 * Updates option for plugin update check when hook for update is triggered
	 */
	public function update_option_for_plugin_update_check() {
		update_site_option(self::UPDATE_OPTION_CHECKED_KEY, self::UNCHECKED);
	}

	/**
	 * Initializes the plugin.
	 */
	private function initialize() {
		Initializer::register_services();
		$this->load_plugin_init_hooks();

		if ( Shop_Helper::is_plugin_enabled() ) {
			$this->load_clever_reach_admin_menu();
			$this->load_clever_reach_newsletter_field();
			$this->load_plugin_text_domain();
			$hook_handler = new Hook_Handler();
			try {
				$hook_handler->register_hooks();
			} catch ( Components\Exceptions\Unable_To_Create_Hook_Handler_Exception $e ) {
				Logger::logError( $e->getMessage(), 'Integration' );
			}
		}

		if ( $this->should_show_notification() ) {
			add_action( 'admin_notices', array( $this, 'show_message' ) );
		}
	}

	/**
	 * Shows admin notification.
	 */
	public function show_message() {
		$notification_data = $this->get_config_service()->get_admin_notification_data();
		$link_label        = __( 'Re-authenticate now', 'cleverreach-wc' );
		echo "
			<div class='notice notice-info is-dismissible'>
				<p>
					<b>{$notification_data['title']}</b>&nbsp;
					{$notification_data['description']}
					<a href='{$notification_data['url']}'>{$link_label}</a>
				</p>
			</div>";
	}

	/**
	 * Registers install and uninstall hook.
	 */
	private function load_plugin_init_hooks() {
		register_activation_hook( $this->cleverreach_plugin_file, array( $this, 'activate' ) );
		add_action( 'init', array( $this, 'start_session' ) );
		add_action( 'admin_init', array( $this, 'initialize_new_site' ) );
		add_action( 'upgrader_process_complete', array( $this, 'update_option_for_plugin_update_check' ), 100, 2 );
		add_filter( 'query_vars', array( $this, 'plugin_add_trigger' ) );
		add_action( 'template_redirect', array( $this, 'plugin_trigger_check' ) );
		add_action( 'plugins_loaded', array( $this, 'load_plugin_text_domain' ) );
		add_action( 'wp_logout', array( $this, 'end_session' ) );
		add_action( 'wp_login', array( $this, 'end_session' ) );
		if ( is_multisite() ) {
			add_action( 'delete_blog', array( $this, 'uninstall_plugin_from_deleted_site' ) );
		}
	}

	/**
	 * Adds CleverReach item to backend administrator menu.
	 */
	private function load_clever_reach_admin_menu() {
		if ( is_admin() && ! is_network_admin() ) {
			add_action( 'admin_menu', array( $this, 'create_admin_menu' ) );
		}
	}

	/**
	 * Adds newsletter subscription field to user forms.
	 */
	private function load_clever_reach_newsletter_field() {
		if ( ! Shop_Helper::is_cleverreach_plugin_for_wordpress_installed() ) {
			add_action( 'user_new_form', array( $this, 'register_form_field_newsletter' ) );
			add_action( 'show_user_profile', array( $this, 'register_form_field_newsletter' ) );
			add_action( 'edit_user_profile', array( $this, 'register_form_field_newsletter' ) );
		}

		add_action( 'woocommerce_edit_account_form', array( $this, 'register_form_field_newsletter' ) );
		add_action( 'woocommerce_after_checkout_billing_form', array( $this, 'register_checkout_form_newsletter' ) );
	}

	/**
	 * Checks if plugin was already installed and initialized.
	 *
	 * @return bool
	 */
	private function plugin_already_initialized() {
		$handler = new Database( $this->db );

		return $handler->plugin_already_initialized();
	}

	/**
	 * Initializes plugin database.
	 */
	private function init_database() {
		$installer = new Database( $this->db );
		$installer->install();
	}

	/**
	 * Initializes default configuration values.
	 */
	private function init_config() {
		try {
			$this->get_config_service()->setTaskRunnerStatus( '', null );
		} catch ( TaskRunnerStatusStorageUnavailableException $e ) {
			Logger::logError( $e->getMessage(), 'Integration' );
		}

		$this->get_config_service()->setProductSearchEndpointPassword( md5( time() ) );
		$this->get_config_service()->set_language( Shop_Helper::get_shop_language() );
		$this->get_config_service()->set_database_version( Shop_Helper::get_plugin_version() );
		$this->get_config_service()->set_plugin_updated_to_version_1_2_0();
	}

	/**
	 * Handles app uninstall.
	 */
	private function uninstall_handler() {
		$config_service = $this->get_config_service();
		$proxy          = $this->get_proxy();
		try {
			$proxy->deleteProductSearchEndpoint( $config_service->getProductSearchContentId() );
		} catch ( \Exception $e ) {
			Logger::logError( 'Could not delete product search endpoint because: ' . $e->getMessage(), 'Integration' );
		}

		try {
			$proxy->deleteReceiverEvent();
		} catch ( \Exception $e ) {
			Logger::logError( 'Could not delete receiver event because: ' . $e->getMessage(), 'Integration' );
		}

		/** @var AuthProxy $auth_proxy */
		$auth_proxy = ServiceRegister::getService( AuthProxy::CLASS_NAME );
		$auth_proxy->revokeOAuth();
	}

	/**
	 * Returns whether CleverReach notification should be shown.
	 *
	 * @return bool
	 */
	private function should_show_notification() {
		return ! $this->is_cleverreach_page() && $this->get_config_service()->show_admin_notice();
	}

	/**
	 * Returns whether the current page is a CleverReach plugin page.
	 *
	 * @return bool
	 */
	private function is_cleverreach_page() {
		return array_key_exists( 'page', $_REQUEST ) && ( 'cleverreach-wc' === $_REQUEST[ 'page' ] );
	}

	/**
	 * Retrieves proxy.
	 *
	 * @return ConcreteProxy
	 */
	private function get_proxy() {
		if ( null === $this->proxy ) {
			$this->proxy = ServiceRegister::getService( Proxy::CLASS_NAME );
		}

		return $this->proxy;
	}

	/**
	 * Retrieves config service;
	 *
	 * @return Config_Service
	 */
	private function get_config_service() {
		if ( null === $this->config_service ) {
			$this->config_service = ServiceRegister::getService( Configuration::CLASS_NAME );
		}

		return $this->config_service;
	}

	/**
	 * Updates plugin on single WordPress site.
	 */
	private function update_plugin_on_single_site() {
		if ( Shop_Helper::is_plugin_enabled() && $this->is_plugin_version_changed() ) {
			try {
				$this->run_migrations();
			} catch ( \Exception $exception ) {
				Logger::logError( $exception->getMessage(), 'Database Update' );
				return false;
			}
		}

		$this->get_config_service()->set_database_version( Shop_Helper::get_plugin_version() );

		return true;
	}

	/**
	 * Check if update script should execute
	 *
	 * @return bool
	 */
	private function is_plugin_version_changed() {
		$previous_version = $this->get_config_service()->get_database_version();
		$current_version  = Shop_Helper::get_plugin_version();
		if ( ! $this->get_config_service()->is_plugin_updated_to_version_1_2_0() ) {
			$this->config_service->set_database_version( '1.1.0' );

			return true;
		}

		return version_compare( $previous_version, $current_version, 'lt' );
	}

	/**
	 * Run all required migrations
	 */
	private function run_migrations() {
		$migration_dir = realpath( __DIR__ ) . '/database/migrations/';
		$previous_version = $this->get_config_service()->get_database_version();
		$version_file_reader = new Version_File_Reader( $migration_dir, $previous_version );
		while ( $version_file_reader->has_next() ) {
			/** @var Update_Schema $update_schema */
			$update_schema = $version_file_reader->read_next();
			$update_schema->update();
		}
	}

	/**
	 * Switches to site with provided ID and removes plugin from that site.
	 *
	 * @param int $site_id Site identifier.
	 */
	private function switch_to_site_and_uninstall_plugin( $site_id ) {
		switch_to_blog( $site_id );

		$this->uninstall_plugin_from_site();

		restore_current_blog();
	}

	/**
	 * Removes plugin tables and configuration from the current site.
	 */
	private function uninstall_plugin_from_site() {
		$installer = new Database( $this->db );
		$this->uninstall_handler();
		$installer->uninstall();
	}
}
