<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Database\Migrations;

use CleverReach\WooCommerce\Components\InfrastructureServices\Config_Service;
use CleverReach\WooCommerce\Components\Utility\Update\Update_Schema;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Sync\AttributesSyncTask;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Queue;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\QueueItem;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Exceptions\QueueStorageUnavailableException;
use CleverReach\WooCommerce\Components\Utility\Database;
use CleverReach\WooCommerce\Components\Repositories\Base_Repository;

/**
 * Class Migration_1_4_0
 *
 * @package CleverReach\WooCommerce\Database\Migrations
 */
class Migration_1_4_0 extends Update_Schema {

	/**
	 * Run update logic for current migration
	 *
	 * @throws \CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Exceptions\QueueStorageUnavailableException
	 */
	public function update() {
		$this->add_priority_column_to_queue_table();
		$this->add_composite_index_to_queue_table();
		$this->update_attributes();
	}

	/**
	 * Adds a new column that signifies queue item priority to the queue table.
	 */
	private function add_priority_column_to_queue_table() {
		$this->db->query( 'ALTER TABLE `' . Base_Repository::get_table_name( Database::QUEUE_TABLE )
		                  . '` ADD `priority` INT(11) NOT NULL DEFAULT ' . QueueItem::PRIORITY_MEDIUM );
	}

	/**
	 * Adds composite index to queue table.
	 */
	private function add_composite_index_to_queue_table() {
		$this->db->query( 'ALTER TABLE `' . Base_Repository::get_table_name( Database::QUEUE_TABLE )
		                  . '` ADD INDEX `cleverreach_queue_idx_status_queuename_priority` (`status`,`queueName`,`priority`)' );
	}

	/**
	 * Updates attributes by enqueuing attributes sync task.
	 *
	 * @throws QueueStorageUnavailableException When we are unable to enqueue task.
	 */
	private function update_attributes() {
		/**
		 * Queue service
		 *
		 * @var Queue $queue_service
		 */
		$queue_service = ServiceRegister::getService( Queue::CLASS_NAME );
		/**
		 * Config service
		 *
		 * @var Config_Service $config_service
		 */
		$config_service = ServiceRegister::getService( Configuration::CLASS_NAME );
		/**
		 * Queue item.
		 *
		 * @var QueueItem $initial_sync_task
		 */
		$initial_sync_task = $queue_service->findLatestByType( 'Initial_Sync_Task' );
		if ( null !== $initial_sync_task && $initial_sync_task->getStatus() === QueueItem::COMPLETED ) {
			$queue_service->enqueue( $config_service->getQueueName(), new AttributesSyncTask() );
		}
	}
}